"""LCS.py

Dynamic program for longest common subsequences

D. Eppstein, March 2002.
"""

def LongestCommonSubsequence(A,B):
    """Find longest common subsequence of iterables A and B."""
    A = list(A)
    B = list(B)

    # Fill dictionary lcsLen[i,j] with length of LCS of A[:i] and B[:j]
    lcsLen = {}
    for i in range(len(A)+1):
        for j in range(len(B) + 1):
            if i == 0 or j == 0:
                lcsLen[i,j] = 0
            elif A[i-1] == B[j-1]:
                lcsLen[i,j] = lcsLen[i-1,j-1] + 1
            else:
                lcsLen[i,j] = max(lcsLen[i-1,j], lcsLen[i,j-1])

    # Produce actual sequence by backtracking through pairs (i,j),
    # using computed lcsLen values to guide backtracking
    i = len(A)
    j = len(B)
    LCS = []
    while lcsLen[i,j]:
        while lcsLen[i,j] == lcsLen[i-1,j]:
            i -= 1
        while lcsLen[i,j] == lcsLen[i,j-1]:
            j -= 1
        i -= 1
        j -= 1
        LCS.append(A[i])

    LCS.reverse()
    return LCS



"""PartitionRefinement.py

Maintain and refine a partition of a set of items into subsets,
as used e.g. in Hopcroft's DFA minimization algorithm,
modular decomposition of graphs, etc.

D. Eppstein, November 2003.
"""

class PartitionError(Exception): pass

class PartitionRefinement:
    """Maintain and refine a partition of a set of items into subsets.
    Space usage for a partition of n items is O(n), and each refine
    operation takes time proportional to the size of its argument.
    """

    def __init__(self,items):
        """Create a new partition refinement data structure for the given
        items.  Initially, all items belong to the same subset.
        """
        S = set(items)
        self._sets = {id(S):S}
        self._partition = {x:S for x in S}

    def __getitem__(self,element):
        """Return the set that contains the given element."""
        return self._partition[element]

    def __iter__(self):
        """Loop through the sets in the partition."""
        return self._sets.itervalues()

    def __len__(self):
        """Return the number of sets in the partition."""
        return len(self._sets)

    def add(self,element,theset):
        """Add a new element to the given partition subset."""
        if id(theset) not in self._sets:
            raise PartitionError("Set does not belong to the partition")
        if element in self._partition:
            raise PartitionError("Element already belongs to the partition")
        theset.add(element)
        self._partition[element] = theset

    def remove(self,element):
        """Remove the given element from its partition subset."""
        self._partition[element].remove(element)
        del self._partition[element]

    def refine(self,S):
        """Refine each set A in the partition to the two sets
        A & S, A - S.  Return a list of pairs (A & S, A - S)
        for each changed set.  Within each pair, A & S will be
        a newly created set, while A - S will be a modified
        version of an existing set in the partition.
        Not a generator because we need to perform the partition
        even if the caller doesn't iterate through the results.
        """
        hit = {}
        output = []
        for x in S:
            if x in self._partition:
                Ax = self._partition[x]
                hit.setdefault(id(Ax),set()).add(x)
        for A,AS in hit.items():
            A = self._sets[A]
            if AS != A:
                self._sets[id(AS)] = AS
                for x in AS:
                    self._partition[x] = AS
                A -= AS
                output.append((AS,A))
        return output

    def freeze(self):
        """Make all sets in S immutable."""
        for S in self._sets.values():
            F = frozenset(S)
            for x in F:
                self._partition[x] = F
            self._sets[id(F)] = F
            del self._sets[id(S)]



"""SVG.py

Simple generation of SVG-format vector graphics files

D. Eppstein, November 2011.
"""

def _coord(x):
    """String representation for coordinate"""
    return ("%.4f" % x).rstrip("0").rstrip(".")

class SVG:
    def __init__(self, bbox, stream,
               standalone=True, prefix=None, indentation=0):
        """Create a new SVG object, to be written to the given stream.
        If standalone is True or omitted, the SVG object becomes a whole
        XML file; otherwise, it becomes an XML object within a larger XML
        file. If the prefix is nonempty, it is used to distinguish svg tags
        from other tags; a reasonable choice for the prefix value would be
        "s" or "svg". If the indentation is nonzero, it gives a number of
        spaces by which every line of the file is indented.
        
        The bbox argument should be a complex number, the farthest visible
        point from the origin in the positive quadrant. The bounding box
        will become the rectangle between the origin and that point.
        All other methods that specify points should do so using
        complex number coordinates."""
        self.stream = stream
        if prefix:
            self.prefix = prefix + ":"
        else:
            self.prefix = ""
        if standalone:
            print >>stream,'''<?xml version="1.0" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" 
  "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">'''
        self.indentation = indentation
        self.nesting = 0
        br = _coord(bbox.real)
        bi = _coord(bbox.imag)
        self.element('''svg width="%s" height="%s" viewBox="0 0 %s %s"
     xmlns="http://www.w3.org/2000/svg" version="1.1"''' % (br,bi,br,bi),+1)

    def close(self):
        """Output the end of an SVG file."""
        self.element("svg", -1)
        if self.nesting:
            raise Exception("SVG: Unclosed tags")

    def element(self, e, delta=0, style={}, **morestyle):
        """Output an SVG element.
        The delta argument distinguishes between XML tags that
        open a nested section of the XML file (delta=+1), XML tags
        that close the same section (delta=-1), and XML tags that
        stand alone without anything nested inside them (delta=0).
        Every call with delta=+1 must be matched by a call with delta=-1.
        If the style argument is nonempty, it should be a dictionary
        of style parameters, included within the object; these
        may also be passed as keyword arguments to element.
        If the same keyword is present both in style and as a keyword
        argument, the keyword argument takes priority."""
        if delta < 0:
            self.nesting += delta
        output = [" " * (self.indentation + 2*self.nesting), "<"]
        if delta < 0:
            output.append("/")
        output += [self.prefix, e]
        style = dict(style)
        style.update(morestyle)
        if style:
            output.append(' style="')
            second = False
            for keyword in style:
                if second:
                    output.append("; ")
                second = True
                output += [keyword, ":", style[keyword]]
            output.append('"')
        if delta > 0:
            self.nesting += delta
        elif delta == 0:
            output.append("/")
        output.append(">")
        print "".join(output)

    def group(self,style={},**morestyle):
        """Start a group of objects, all with the same style"""
        self.element("g", +1, style=style, **morestyle)

    def ungroup(self):
        """End a group of objects"""
        self.element("g", -1)

    def circle(self, center, radius, style={}, **morestyle):
        """Circle with given center and radius"""
        self.element('circle cx="%s" cy="%s" r="%s"' %
                (_coord(center.real), _coord(center.imag), _coord(radius)),
            style=style, **morestyle)

    def segment(self, p, q, style={}, **morestyle):
        """Line segment from p to q"""
        self.element('line x1="%s" y1="%s" x2="%s" y2="%s"' % 
                     (_coord(p.real), _coord(p.imag),
                      _coord(q.real), _coord(q.imag)), style=style, **morestyle)

    def arc(self, p, q, r, large=False, style={}, **morestyle):
        """Circular arc from p to q with radius r.
        If the large flag is set true, the arc will cover more than
        half of a circle. The SVG "sweep-flag" is not provided;
        instead, to achieve the same effect, swap p and q."""
        if large:
            large = "1"
        else:
            large = "0"
        r = _coord(r)
        self.element('path d="M %s,%s A %s,%s 0 %s 0 %s,%s"' %
                     (_coord(p.real),_coord(p.imag),r,r,large,
                      _coord(q.real),_coord(q.imag)), style=style, **morestyle)

# A small color palette chosen to have high contrast
# even when viewed by color-blind readers
class colors:
    none = "none"
    white = "#FFFFFF"
    black = "#000000"
    red = "#BC1E46"
    blue = "#0081CD"
    green = "#009246"
    yellow = "FEC200"
    magenta = "CC33CC"



"""LCA.py

Range minimization and tree least common ancestor data structures
with linear space and preprocessing time, and constant query time,
from Bender and Farach-Colton, "The LCA Problem Revisited",
Proc. LATIN 2000 (pp.88-94), http://www.cs.sunysb.edu/~bender/pub/lca.ps

Some experimentation would be needed to determine how large a query
range needs to be to make this faster than computing the min of the range
directly, and how much input data is needed to make the linear space
version pay off compared to the much simpler LogarithmicRangeMin that
it uses as a subroutine.

D. Eppstein, November 2003.
"""

import unittest,random
from collections import defaultdict
from UnionFind import UnionFind

def _decodeSlice(self,it):
    """Work around removal of __getslice__ in Python 3"""
    if type(it) != slice:
        raise ValueError("Can only access LCA object by slice notation")
    left,right,stride = it.indices(len(self))
    if stride != 1:
        raise ValueError("Stride not permitted in LCA")
    return left,right

class RangeMin:
    """If X is any list, RangeMin(X)[i:j] == min(X[i:j]).
    Initializing RangeMin(X) takes time and space linear in len(X),
    and querying the minimum of a range takes constant time per query.
    """

    def __init__(self,X):
        """Set up structure with sequence X as data.
        Uses an LCA structure on a Cartesian tree for the input."""
        self._data = list(X)
        if len(X) > 1:
            big = map(max, self._ansv(False), self._ansv(True))
            parents = {i:big[i][1] for i in range(len(X)) if big[i]}
            self._lca = LCA(parents)

    def __getitem__(self,it):
        """When called by X[left:right], return min(X[left:right])."""
        left,right = _decodeSlice(self,it)
        if right <= left:
            return None     # empty range has no minimum
        return self._data[self._lca(left,right-1)]

    def __len__(self):
        """How much data do we have?  Needed for negative index in slice."""
        return len(self._data)

    def _ansv(self,reversed):
        """All nearest smaller values.
        For each x in the data, find the value smaller than x in the closest
        position to the left of x (if not reversed) or to the right of x
        (if reversed), and return list of pairs (smaller value,position).
        Due to our use of positions as a tie-breaker, values equal to x
        count as smaller on the left and larger on the right.
        """
        stack = [(min(self._data),-1)]   # protect stack top with sentinel
        output = [0]*len(self._data)
        for xi in _pairs(self._data,reversed):
            while stack[-1] > xi:
                stack.pop()
            output[xi[1]] = stack[-1]
            stack.append(xi)
        return output

    def _lca(self,first,last):
        """Function to replace LCA when we have too little data."""
        return 0

class RestrictedRangeMin:
    """Linear-space RangeMin for integer data obeying the constraint
        abs(X[i]-X[i-1])==1.
    We don't actually check this constraint, but results may be incorrect
    if it is violated.  For the use of this data structure from LCA, the
    data are actually pairs rather than integers, but the minima of
    all ranges are in the same positions as the minima of the integers
    in the first positions of each pair, so the data structure still works.
    """
    def __init__(self,X):
        # Compute parameters for partition into blocks.
        # Position i in X becomes transformed into
        # position i&self._blockmask in block i>>self.blocklen
        self._blocksize = _log2(len(X))//2
        self._blockmask = (1 << self._blocksize) - 1
        blocklen = 1 << self._blocksize

        # Do partition into blocks, find minima within
        # each block, prefix minima in each block,
        # and suffix minima in each block
        blocks = []             # map block to block id
        ids = {}                # map block id to PrecomputedRangeMin
        blockmin = []           # map block to min value
        self._prefix = [None]   # map data index to prefix min of block
        self._suffix = []       # map data index to suffix min of block
        for i in range(0,len(X),blocklen):
            XX = X[i:i+blocklen]
            blockmin.append(min(XX))
            self._prefix += PrefixMinima(XX)
            self._suffix += PrefixMinima(XX,reversed=True)
            blockid = len(XX) < blocklen and -1 or self._blockid(XX)
            blocks.append(blockid)
            if blockid not in ids:
                ids[blockid] = PrecomputedRangeMin(_pairs(XX))
        self._blocks = [ids[b] for b in blocks]

        # Build data structure for interblock queries
        self._blockrange = LogarithmicRangeMin(blockmin)
        self._data = list(X)

    def __len__(self):
        """How much data do we have?  Needed for negative index in slice."""
        return len(self._data)

    def __getitem__(self,it):
        """When called by X[left:right], return min(X[left:right])."""
        left,right = _decodeSlice(self,it)
        firstblock = left >> self._blocksize
        lastblock = (right - 1) >> self._blocksize
        if firstblock == lastblock:
            i = left & self._blockmask
            position = self._blocks[firstblock][i:i+right-left][1]
            return self._data[position + (firstblock << self._blocksize)]
        else:
            best = min(self._suffix[left], self._prefix[right])
            if lastblock > firstblock + 1:
                best = min(best, self._blockrange[firstblock+1:lastblock])
            return best

    def _blockid(self,XX):
        """Return value such that all blocks with the same
        pattern of increments and decrements get the same id.
        """
        blockid = 0
        for i in range(1,len(XX)):
            blockid = blockid*2 + (XX[i] > XX[i-1])
        return blockid

class PrecomputedRangeMin:
    """RangeMin solved in quadratic space by precomputing all solutions."""

    def __init__(self,X):
        self._minima = [PrefixMinima(X[i:]) for i in range(len(X))]

    def __getitem__(self,it):
        """When called by X[left:right], return min(X[left:right])."""
        left,right = _decodeSlice(self,it)
        return self._minima[left][right-left-1]

    def __len__(self):
        return len(self._minima)

class LogarithmicRangeMin:
    """RangeMin in O(n log n) space and constant query time."""

    def __init__(self,X):
        """Compute min(X[i:i+2**j]) for each possible i,j."""
        self._minima = m = [list(X)]
        for j in range(_log2(len(X))):
            m.append(map(min, m[-1][:-1<<j], m[-1][1<<j:]))

    def __getitem__(self,it):
        """When called by X[left:right], return min(X[left:right])."""
        left,right = _decodeSlice(self,it)
        j = _logtable[right-left]
        row = self._minima[j]
        return min(row[left],row[right-2**j])

    def __len__(self):
        return len(self._minima[0])

class LCA:
    """Structure for finding least common ancestors in trees.
    Tree nodes may be any hashable objects; a tree is specified
    by a dictionary mapping nodes to their parents.
    LCA(T)(x,y) finds the LCA of nodes x and y in tree T.
    """
    def __init__(self, parent, RangeMinFactory = RestrictedRangeMin):
        """Construct LCA structure from tree parent relation."""
        children = defaultdict(list)
        for x in parent:
            children[parent[x]].append(x)
        root = [x for x in children if x not in parent]
        if len(root) != 1:
            raise ValueError("LCA input is not a tree")

        levels = []
        self._representatives = {}
        self._visit(children,levels,root[0],0)
        if [x for x in parent if x not in self._representatives]:
            raise ValueError("LCA input is not a tree")
        self._rangemin = RangeMinFactory(levels)

    def __call__(self,*nodes):
        """Find least common ancestor of a set of nodes."""
        r = [self._representatives[x] for x in nodes]
        return self._rangemin[min(r):max(r)+1][1]

    def _visit(self,children,levels,node,level):
        """Perform Euler traversal of tree."""
        self._representatives[node] = len(levels)
        pair = (level,node)
        levels.append(pair)
        for child in children[node]:
            self._visit(children,levels,child,level+1)
            levels.append(pair)

class OfflineLCA(defaultdict):
    """Find LCAs of all pairs in a given sequence, using Union-Find."""

    def __init__(self,parent,pairs):
        """Set up to find LCAs of pairs in tree defined by parent.
        LCA of any given pair x,y can then be found by self[x][y].
        However unlike the online LCA structure we can not find LCAs
        of pairs that are not supplied to us at startup time.
        """

        # set up dictionary where answers get stored
        defaultdict.__init__(self,dict)
        for u,v in pairs:
            self[u][v] = self[v][u] = None

        # set up data structure for finding node ancestor on search path
        # self.descendants forms a collection of disjoint sets,
        #    one set for the descendants of each search path node.
        # self.ancestors maps disjoint set ids to the ancestors themselves.
        self.descendants = UnionFind()
        self.ancestors = {}

        # invert the parent relationship so we can traverse the tree
        self.children = defaultdict(list)
        for x,px in parent.iteritems():
            self.children[px].append(x)
        root = [x for x in self.children if x not in parent]
        if len(root) != 1:
            raise ValueError("LCA input is not a tree")

        # initiate depth first traversal
        self.visited = set()
        self.traverse(root[0])

    def traverse(self,node):
        """Perform depth first traversal of tree."""
        self.ancestors[self.descendants[node]] = node
        for child in self.children[node]:
            self.traverse(child)
            self.descendants.union(child,node)
            self.ancestors[self.descendants[node]] = node
        self.visited.add(node)
        for query in self[node]:
            if query in self.visited:
                lca = self.ancestors[self.descendants[query]]
                self[node][query] = self[query][node] = lca

# Various utility functions

def PrefixMinima(X,reversed=False):
    """Compute table of prefix minima
    (or suffix minima, if reversed=True) of list X.
    """
    current = None
    output = [None]*len(X)
    for x,i in _pairs(X,reversed):
        if current is None:
            current = x
        else:
            current = min(current,x)
        output[i] = current
    return output

def _pairs(X,reversed=False):
    """Return pairs (x,i) for x in list X, where i is
    the index of x in the data, in forward or reverse order.
    """
    indices = range(len(X))
    if reversed:
        indices.reverse()
    return [(X[i],i) for i in indices]

_logtable = [None,0]
def _log2(n):
    """Make table of logs reach up to n and return floor(log_2(n))."""
    while len(_logtable) <= n:
        _logtable.extend([1+_logtable[-1]]*len(_logtable))
    return _logtable[n]

# if run as "python LCA.py", run tests on random data
# and check that RangeMin's results are correct.

class RandomRangeMinTest(unittest.TestCase):
    def testRangeMin(self):
        for trial in range(20):
            data = [random.choice(xrange(1000000))
                    for i in range(random.randint(1,100))]
            R = RangeMin(data)
            for sample in range(100):
                i = random.randint(0,len(data)-1)
                j = random.randint(i+1,len(data))
                self.assertEqual(R[i:j],min(data[i:j]))

class LCATest(unittest.TestCase):
    parent = {'b':'a','c':'a','d':'a','e':'b','f':'b','g':'f','h':'g','i':'g'}
    lcas = {
        ('a','b'):'a',
        ('b','c'):'a',
        ('c','d'):'a',
        ('d','e'):'a',
        ('e','f'):'b',
        ('e','g'):'b',
        ('e','h'):'b',
        ('c','i'):'a',
        ('a','i'):'a',
        ('f','i'):'f',
    }

    def testLCA(self):
        L = LCA(self.parent)
        for k,v in self.lcas.iteritems():
            self.assertEqual(L(*k),v)

    def testLogLCA(self):
        L = LCA(self.parent, LogarithmicRangeMin)
        for k,v in self.lcas.iteritems():
            self.assertEqual(L(*k),v)

    def testOfflineLCA(self):
        L = OfflineLCA(self.parent, self.lcas.iterkeys())
        for (p,q),v in self.lcas.iteritems():
            self.assertEqual(L[p][q],v)

if __name__ == "__main__":
    unittest.main()   




"""GraphExamples.py

Various examples of small standard named graphs.

D. Eppstein, September 2005.
"""

def GeneralizedPetersenGraph(n,k):
    G = {}
    for i in range(n):
        G[i,True] = (i,False),((i-1)%n,True),((i+1)%n,True)
        G[i,False] = (i,True),((i-k)%n,False),((i+k)%n,False)
    return G

PetersenGraph = GeneralizedPetersenGraph(5,2)
DesarguesGraph = GeneralizedPetersenGraph(10,3)

def GeneralizedCoxeterGraph(n,a,b):
    G = {}
    for i in range(n):
        G[i,0] = (i,1),(i,2),(i,3)
        G[i,1] = (i,0),((i+1)%n,1),((i-1)%n,1)
        G[i,2] = (i,0),((i+a)%n,2),((i-a)%n,2)
        G[i,3] = (i,0),((i+b)%n,1),((i-b)%n,1)
    return G

CoxeterGraph = GeneralizedCoxeterGraph(7,2,3)

def CubeConnectedCycles(n):
    return {(x,y):[(x,(y+1)%n),(x,(y-1)%n),(x^(1<<y),y)]
            for x in range(1<<n) for y in range(n)}

def LCFNotation(L,n):
    """
    Construct the cubic Hamiltonian graph with LCF Notation L^n.
    See http://mathworld.wolfram.com/LCFNotation.html
    for a description of this notation.
    """
    n *= len(L)
    return {i:((i-1)%n,(i+1)%n,(i+L[i%len(L)])%n) for i in range(n)}

McGeeGraph = LCFNotation([-12,7,-7],8)
DyckGraph = LCFNotation([-13,5,-5,13],8)
PappusGraph = LCFNotation([5,7,-7,7,-7,-5],3)
TutteCoxeterGraph = LCFNotation([-13,-9,7,-7,9,13],5)
GrayGraph = LCFNotation([-25,7,-7,13,-13,25],9)



"""BFS.py

Breadth First Search. See also LexBFS.py.

D. Eppstein, May 2007.
"""

def BreadthFirstLevels(G,root):
    """
    Generate a sequence of bipartite directed graphs, each consisting
    of the edges from level i to level i+1 of G. Edges that connect
    vertices within the same level are not included in the output.
    The vertices in each level can be listed by iterating over each
    output graph.
    """
    visited = set()
    currentLevel = [root]
    while currentLevel:
        for v in currentLevel:
            visited.add(v)
        nextLevel = set()
        levelGraph = {v:set() for v in currentLevel}
        for v in currentLevel:
            for w in G[v]:
                if w not in visited:
                    levelGraph[v].add(w)
                    nextLevel.add(w)
        yield levelGraph
        currentLevel = nextLevel



"""IntegerHeap.py

Priority queues of integer keys based on van Emde Boas trees.
Only the keys are stored; caller is responsible for keeping
track of any data associated with the keys in a separate dictionary.

We use a version of vEB trees in which all accesses to subtrees
are performed indirectly through a hash table and the data structures
for the subtrees are only created when they are nonempty. As a
consequence, the data structure takes only linear space
(linear in the number of keys stored in the heap) while still preserving
the O(log log U) time per operation of vEB trees. For better performance,
we switch to bitvectors for sufficiently small integer sizes.

Usage:
    Q = BitVectorHeap() # Bit-vector based heap for integers
    Q = FlatHeap(i)     # Flat heap for 2^i-bit integers
    Q = LinearHeap()    # Set-based heap with linear-time min operation
    Q = IntegerHeap(i)  # Choose between BVH and FH depending on i
    Q.add(x)            # Include x among the values in the heap
    Q.remove(x)         # Remove x from the values in the heap
    Q.min()             # Return the minimum value in the heap
    if Q                # True if Q is nonempty, false if empty

Because the min operation in LinearHeap is a Python primitive rather than
a sequence of interpreted Python instructions, it is actually quite fast;
testing indicates that, for 32-bit keys, FlatHeap(5) beats LinearHeap only
for heaps of 250 or more items. This breakeven point would likely be
different for different numbers of bits per word or when runtime optimizers
such as psyco are in use.

D. Eppstein, January 2010
"""

def IntegerHeap(i):
    """Return an integer heap for 2^i-bit integers.
    We use a BitVectorHeap for small i and a FlatHeap for large i.
    
    Timing tests indicate that the cutoff i <= 3 is slightly
    faster than the also-plausible cutoff i <= 2, and that both
    are much faster than the way-too-large cutoff i <= 4.
    The resulting IntegerHeap objects will use 255-bit long integers,
    still small compared to the overhead of a FlatHeap."""
    if i <= 3:
        return BitVectorHeap()
    return FlatHeap(i)

Log2Table = {}          # Table of powers of two, with their logs
def Log2(b):
    """Return log_2(b), where b must be a power of two."""
    while b not in Log2Table:
        i = len(Log2Table)
        Log2Table[1<<i] = i
    return Log2Table[b]

# ======================================================================
#   BitVectorHeap
# ======================================================================

class BitVectorHeap:
    """Maintain the minimum of a set of integers using bitvector operations."""
    def __init__(self):
        """Create a new BitVectorHeap."""
        self._S = 0L
        
    def __nonzero__(self):
        """True if this heap is nonempty, false if empty."""
        return self._S != 0

    def add(self,x):
        """Include x among the values in the heap."""
        self._S |= 1<<x

    def remove(self,x):
        """Remove x from the values in the heap."""
        self._S &=~ 1<<x

    def min(self):
        """Return the minimum value in the heap."""
        if not self._S:
            raise ValueError("BitVectorHeap is empty")
        return Log2(self._S &~ (self._S - 1))

# ======================================================================
#   FlatHeap
# ======================================================================

class FlatHeap:
    """Maintain the minimum of a set of 2^i-bit integer values."""
    def __init__(self,i):
        """Create a new FlatHeap for 2^i-bit integers."""
        self._min = None
        self._order = i
        self._shift = (1 << (i - 1))
        self._max = (1 << (1 << i)) - 1
        self._HQ = IntegerHeap(i-1) # Heap of high halfwords
        self._LQ = {}               # Map high half to heaps of low halfwords

    def _rangecheck(self,x):
        """Make sure x is a number we can include in this FlatHeap."""
        if x < 0 or x > self._max:
            raise ValueError("FlatHeap: %s out of range" % repr(x))

    def __nonzero__(self):
        """True if this heap is nonempty, false if empty."""
        return not (self._min is None)

    def min(self):
        """Return the minimum value in the heap."""
        if self._min is None:
            raise ValueError("FlatHeap is empty")
        return self._min

    def add(self,x):
        """Include x among the values in the heap."""
        self._rangecheck(x)
        if self._min is None or self._min == x:
            # adding to an empty heap is easy
            self._min = x
            return
        if x < self._min:
            # swap to make sure the value we're adding is non-minimal
            x, self._min = self._min, x
        H = x >> self._shift            # split into high and low halfwords
        L = x - (H << self._shift)
        if H not in self._LQ:
            self._HQ.add(H)
            self._LQ[H] = IntegerHeap(self._order-1)
        self._LQ[H].add(L)

    def remove(self,x):
        """Remove x from the values in the heap."""
        self._rangecheck(x)
        if self._min == x:
            # Removing minimum, move next value into place
            # and prepare to remove that next value from secondary heaps
            if not self._HQ:
                self._min = None
                return
            H = self._HQ.min()
            L = self._LQ[H].min()
            x = self._min = (H << self._shift) + L
        else:
            H = x >> self._shift            # split into high and low halfwords
            L = x - (H << self._shift)
        if H not in self._LQ:
            return                          # ignore removal when not in heap
        self._LQ[H].remove(L)
        if not self._LQ[H]:
            del self._LQ[H]
            self._HQ.remove(H)

# ======================================================================
#   LinearHeap
# ======================================================================


class LinearHeap:
    """Maintain the minimum of a set of integers using a set object."""
    def __init__(self):
        """Create a new BitVectorHeap."""
        self._S = set()
        
    def __nonzero__(self):
        """True if this heap is nonempty, false if empty."""
        return len(self._S) > 0

    def add(self,x):
        """Include x among the values in the heap."""
        self._S.add(x)

    def remove(self,x):
        """Remove x from the values in the heap."""
        self._S.remove(x)

    def min(self):
        """Return the minimum value in the heap."""
        return min(self._S)


# ======================================================================
#   Unit tests
# ======================================================================

if __name__ == "__main__":
    import unittest,random
    random.seed(1234)

    class IntegerHeapTest(unittest.TestCase):
        def testHeaps(self):
            o = 5               # do tests on 2^5-bit integers
            N = LinearHeap()
            I = IntegerHeap(o)
            for iteration in xrange(20000):
                self.assertEqual(bool(N),bool(I))   # both have same emptiness
                if (not N) or random.randrange(2):  # flip coin for add/remove
                    x = random.randrange(1<<(1<<o))
                    N.add(x)
                    I.add(x)
                else:
                    x = N.min()
                    self.assertEqual(x,I.min())
                    N.remove(x)
                    I.remove(x)

    print "Testing IntegerHeap.py"
    unittest.main()   



"""AcyclicReachability.py

Bit-parallel algorithm for testing which vertices can reach which other vertices in a DAG.

Usage:
    R = Reachability(G)
    ...
    R.reachable(source,destination)
returns a boolean value: True if G contains a path from the source vertex
to the destination vertex, and False otherwise. The initialization of R
performs a linear number of bitvector operations, after which each reachability
test takes constant time to perform.

D. Eppstein, April 2009.
"""

import unittest
from PartialOrder import TopologicalOrder

class Reachability:
    def __init__(self,G):
        """Initialize a reachability data structure for the given DAG."""
        self.key = {}
        self.canReach = []
        L = TopologicalOrder(G)
        L.reverse()
        for v in L:
            k = self.key[v] = len(self.canReach)
            bits = 1L<<k
            for w in G[v]:
                bits |= self.canReach[self.key[w]]
            self.canReach.append(bits)

    def reachable(self,source,destination):
        """Test whether the DAG has a path from source to destination."""
        return (1L<<self.key[destination])&self.canReach[self.key[source]] != 0

class ReachabilityTest(unittest.TestCase):
    def testReachable(self):
        G = {"A":["C"],"B":["C","D"],"C":["D","E"],"D":[],"E":[]}
        R = Reachability(G)
        for s in "ABCDE":
            for t in "ABCDE":
                self.assertEqual(R.reachable(s,t),
                                 s <= t and s+t not in ["AB","DE"])

if __name__ == "__main__":
    unittest.main()   



"""Not.py

Symbolic negation operator.

For any Python value x, Not(x) is an object, with the properties that
    Not(x)==Not(y) iff x==y
    Not(Not(x))==x.
If x is hashable, so is Not(x).

The purpose of this is to use within TwoSatisfiability and similar code,
to allow Python objects to represent logical variables. If x is an object representing a variable, Not(x) can be used to represent its negation.

To determine whether a given object y is of the form Not(x), use
isinstance(y,SymbolicNegation).  If it is, you can recover x as Not(y).

D. Eppstein, April 2009.
"""

import unittest

class DoubleNegationError(Exception): pass

class SymbolicNegation:
    def __init__(self,x):
        if isinstance(x,SymbolicNegation):
            raise DoubleNegationError(
              "Use Not(x) rather than instantiating SymbolicNegation directly")
        self.negation = x

    def negate(self):
        return self.negation
        
    def __repr__(self):
        return "Not(" + repr(self.negation) + ")"

    def __eq__(self,other):
        return isinstance(other,SymbolicNegation) and \
               self.negation == other.negation

    def __hash__(self):
        return -hash(self.negation)

def Not(x):
    if isinstance(x,SymbolicNegation):
        return x.negate()
    else:
        return SymbolicNegation(x)

class NotNotTest(unittest.TestCase):
    things = [None,3,"ABC",Not(27)]
    def testNot(self):
        for x in NotNotTest.things:
            self.assertEqual(Not(Not(x)),x)
    def testEq(self):
        for x in NotNotTest.things:
            for y in NotNotTest.things:
                self.assertEqual(Not(x)==Not(y),x==y)
    def testHash(self):
        D = {Not(x):x for x in NotNotTest.things}
        for x in NotNotTest.things:
            self.assertEqual(D[Not(x)],x)

if __name__ == "__main__":
    unittest.main()   



"""StrongConnectivity.py

DFS-based algorithm for computing strongly connected components.

If G is a graph, then
- StronglyConnectedComponents(G) returns a list of
  its components, each represented as a subgraph of G
- Condensation(G) returns a directed acyclic graph, the
  vertices of which are strongly connected components of G.
  Each vertex of the condensation is represented as a frozenset
  of the vertices of G within a single strongly connected component.

D. Eppstein, July 2005.
"""

import unittest
import DFS

class StronglyConnectedComponents(DFS.Searcher):
    """
    Generate the strongly connected components of G.  G should be
    represented in such a way that "for v in G" loops through the
    vertices, and "G[v]" produces a list of the neighbors of v;
    for instance, G may be a dictionary mapping each vertex to its
    neighbor set.  The result of StronglyConnectedComponents(G) is
    a sequence of subgraphs of G.
    """

    def __init__(self,G):
        """Search for strongly connected components of graph G."""

        # set up data structures for DFS
        self._components = []
        self._dfsnumber = {}
        self._activelen = {}
        self._active = []
        self._low = {}
        self._biglow = len(G)
        self._graph = G

        # perform the Depth First Search
        DFS.Searcher.__init__(self,G)

        # clean up now-useless data structures
        del self._dfsnumber, self._activelen, self._active, self._low

    def __iter__(self):
        """Return iterator for sequence of strongly connected components."""
        return iter(self._components)

    def __len__(self):
        """How many components are there?"""
        return len(self._components)

    def _component(self,vertices):
        """Make a new SCC."""
        vertices = set(vertices)
        induced = {v:{w for w in self._graph[v] if w in vertices} for v in vertices}
        self._components.append(induced)

    def preorder(self,parent,child):
        """Handle first visit to vertex in DFS search for components."""
        if parent == child:
            self._active = []
        self._activelen[child] = len(self._active)
        self._active.append(child)
        self._low[child] = self._dfsnumber[child] = len(self._dfsnumber)

    def backedge(self,source,destination):
        """Handle non-tree edge in DFS search for components."""
        self._low[source] = min(self._low[source],self._low[destination])

    def postorder(self,parent,child):
        """Handle last visit to vertex in DFS search for components."""
        if self._low[child] == self._dfsnumber[child]:
            self._component(self._active[self._activelen[child]:])
            for v in self._components[-1]:
                self._low[v] = self._biglow
            del self._active[self._activelen[child]:]
        else:
            self._low[parent] = min(self._low[parent],self._low[child])

def Condensation(G):
    """Return a DAG with vertices equal to sets of vertices in SCCs of G."""
    components = {}
    GtoC = {}
    for C in StronglyConnectedComponents(G):
        C = frozenset(C)
        for v in C:
            GtoC[v] = C
        components[C] = set()
    for v in G:
        for w in G[v]:
            if GtoC[v] != GtoC[w]:
                components[GtoC[v]].add(GtoC[w])
    return components
    
# If run as "python StrongConnectivity.py", run tests on various small graphs
# and check that the correct results are obtained.

class StrongConnectivityTest(unittest.TestCase):

    G1 = { 0:[1], 1:[2,3], 2:[4,5], 3:[4,5], 4:[6], 5:[], 6:[] }
    C1 = [[0],[1],[2],[3],[4],[5],[6]]

    # Work around http://bugs.python.org/issue11796 by using a loop
    # instead of a dict/set comprehension in a class variable initializer
    # should be:
    # Con1 = {frozenset([v]):{frozenset([w]) for w in G1[v]} for v in G1}
    Con1 = {}
    for v in G1:
        Con1[frozenset([v])] = {frozenset([w]) for w in G1[v]}
    
    G2 = { 0:[1], 1:[2,3,4], 2:[0,3], 3:[4], 4:[3] }
    C2 = [[0,1,2],[3,4]]
    f012 = frozenset([0,1,2])
    f34 = frozenset([3,4])
    Con2 = {f012:{f34}, f34:set()}
    
    knownpairs = [(G1,C1),(G2,C2)]

    def testStronglyConnectedComponents(self):
        """Check known graph/component pairs."""
        for (graph,expectedoutput) in self.knownpairs:
            output = [list(C) for C in StronglyConnectedComponents(graph)]
            for component in output:
                component.sort()
            output.sort()
            self.assertEqual(output,expectedoutput)

    def testSubgraph(self):
        """Check that each SCC is an induced subgraph."""
        for (graph,expectedoutput) in self.knownpairs:
            components = StronglyConnectedComponents(graph)
            for C in components:
                for v in C:
                    for w in graph:
                        self.assertEqual(w in graph[v] and w in C, w in C[v])

    def testCondensation(self):
        """Check that the condensations are what we expect."""
        self.assertEqual(Condensation(self.G1),self.Con1)
        self.assertEqual(Condensation(self.G2),self.Con2)

if __name__ == "__main__":
    unittest.main()   




"""Wrap.py

Break paragraphs into lines, attempting to avoid short lines.

We use the dynamic programming idea of Knuth-Plass to find the
optimal set of breaks according to a penalty function that
penalizes short lines quadratically; this can be done in linear
time via the OnlineConcaveMinima algorithm in SMAWK.py.

D. Eppstein, August 2005.
"""

from SMAWK import OnlineConcaveMinima

def wrap(text,                  # string or unicode to be wrapped
         target = 76,           # maximum length of a wrapped line
         longlast = False,      # True if last line should be as long as others
         frenchspacing = False, # Single space instead of double after periods
         measure = len,         # how to measure the length of a word
         overpenalty = 1000,    # penalize long lines by overpen*(len-target)
         nlinepenalty = 1000,   # penalize more lines than optimal
         onewordpenalty = 25,   # penalize really short last line
         hyphenpenalty = 25):   # penalize breaking hyphenated words
    """Wrap the given text, returning a sequence of lines."""

    # Make sequence of tuples (word, spacing if no break, cum.measure).
    words = []
    total = 0
    spacings = [0, measure(' '), measure('  ')]
    for hyphenword in text.split():
        if words:
            total += spacings[words[-1][1]]
        parts = hyphenword.split('-')
        for word in parts[:-1]:
            word += '-'
            total += measure(word)
            words.append((word,0,total))
        word = parts[-1]
        total += measure(word)
        spacing = 1
        if word.endswith('.') and (len(hyphenword) > 2 or
                                   not hyphenword[0].isupper()):
            spacing = 2 - frenchspacing
        words.append((word,spacing,total))

    # Define penalty function for breaking on line words[i:j]
    # Below this definition we will set up cost[i] to be the
    # total penalty of all lines up to a break prior to word i.
    def penalty(i,j):
        if j > len(words): return -i    # concave flag for out of bounds
        total = cost.value(i) + nlinepenalty
        prevmeasure = i and words[i-1][2]
        linemeasure = words[j-1][2] - prevmeasure
        if linemeasure > target:
            total += overpenalty * (linemeasure - target)
        elif j < len(words) or longlast:
            total += (target - linemeasure)**2
        elif i == j-1:
            total += onewordpenalty
        if not words[j-1][1]:
            total += hyphenpenalty
        return total

    # Apply concave minima algorithm and backtrack to form lines
    cost = OnlineConcaveMinima(penalty,0)
    pos = len(words)
    lines = []
    while pos:
        breakpoint = cost.index(pos)
        line = []
        for i in range(breakpoint,pos):
            line.append(words[i][0])
            if i < pos-1 and words[i][1]:
                line.append(' '*words[i][1])
        lines.append(''.join(line))
        pos = breakpoint
    lines.reverse()
    return lines



"""CirclePack.py
Compute circle packings according to the Koebe-Thurston-Andreev theory,
Following a numerical algorithm by C. R. Collins and K. Stephenson,
"A Circle Packing Algorithm", Comp. Geom. Theory and Appl. 2003.
"""

from math import pi,acos,asin,sin,e

tolerance = 1+10e-12    # how accurately to approximate things

# ======================================================
#   The main circle packing algorithm
# ======================================================

def CirclePack(internal,external):
    """Find a circle packing for the given data.
    The two arguments should be dictionaries with disjoint keys; the
    keys of the two arguments are identifiers for circles in the packing.
    The internal argument maps each internal circle to its cycle of
    surrounding circles; the external argument maps each external circle
    to its desired radius. The return function is a mapping from circle
    keys to pairs (center,radius) where center is a complex number."""
    
    # Some sanity checks and preprocessing
    if min(external.values()) <= 0:
        raise ValueError("CirclePack: external radii must be positive")
    radii = dict(external)
    for k in internal:
        if k in external:
            raise ValueError("CirclePack: keys are not disjoint")
        radii[k] = 1

    # The main iteration for finding the correct set of radii
    lastChange = 2
    while lastChange > tolerance:
        lastChange = 1
        for k in internal:
            theta = flower(radii,k,internal[k])
            hat = radii[k]/(1/sin(theta/(2*len(internal[k])))-1)
            newrad = hat * (1/(sin(pi/len(internal[k]))) - 1)
            kc = max(newrad/radii[k],radii[k]/newrad)
            lastChange = max(lastChange,kc)
            radii[k] = newrad

    # Recursively place all the circles
    placements = {}
    k1 = iter(internal).next()  # pick one internal circle
    placements[k1] = 0j         # place it at the origin
    k2 = internal[k1][0]        # pick one of its neighbors
    placements[k2] = radii[k1]+radii[k2] # place it on the real axis
    place(placements,radii,internal,k1)  # recursively place the rest
    place(placements,radii,internal,k2)

    return dict((k,(placements[k],radii[k])) for k in radii)

# ======================================================
#   Invert a collection of circles
# ======================================================

def InvertPacking(packing,center):
    """Invert with specified center"""
    result = {}
    for k in packing:
        z,r = packing[k]
        z -= center
        if z == 0:
            offset = 1j
        else:
            offset = z/abs(z)
        p,q = z-offset*r,z+offset*r
        p,q = 1/p,1/q
        z = (p+q)/2
        r = abs((p-q)/2)
        result[k] = z,r
    return result

def NormalizePacking(packing,k=None,target=1.0):
    """Make the given circle have radius one (or the target if given).
    If no circle is given, the minimum radius circle is chosen instead."""
    if k is None:
        r = min(r for z,r in packing.values())
    else:
        z,r = packing[k]
    s = target/r
    return dict((kk,(zz*s,rr*s)) for kk,(zz,rr) in packing.iteritems())

def InvertAround(packing,k,smallCircles=None):
    """Invert so that the specified circle surrounds all the others.
    Searches for the inversion center that maximizes the minimum radius.
    
    This can be expressed as a quasiconvex program, but in a related
    hyperbolic space, so rather than applying QCP methods it seems
    simpler to use a numerical hill-climbing approach, relying on the
    theory of QCP to tell us there are no local maxima to get stuck in.
    
    If the smallCircles argument is given, the optimization
    for the minimum radius circle will look only at these circles"""
    z,r = packing[k]
    if smallCircles:
        optpack = {k:packing[k] for k in smallCircles}
    else:
        optpack = packing
    q,g = z,r*0.4
    oldrad,ratio = None,2
    while abs(g) > r*(tolerance-1) or ratio > tolerance:
        rr,ignore1,ignore2,q = max(list(testgrid(optpack,k,z,r,q,g)))
        if oldrad:
            ratio = rr/oldrad
        oldrad = rr
        g *= 0.53+0.1j  # rotate so not always axis-aligned
    return InvertPacking(packing,q)  


# ======================================================
#   Utility routines, not for outside callers
# ======================================================

def acxyz(x,y,z):
    """Angle at a circle of radius x given by two circles of radii y and z"""
    try:
        return acos(((x+y)**2 + (x+z)**2 - (y+z)**2)/(2.0*(x+y)*(x+z)))
    except ValueError:
        return pi/3
    except ZeroDivisionError:
        return pi

def flower(radius,center,cycle):
    """Compute the angle sum around a given internal circle"""
    return sum(acxyz(radius[center],radius[cycle[i-1]],radius[cycle[i]])
               for i in range(len(cycle)))

def place(placements,radii,internal,center):
    """Recursively find centers of all circles surrounding k"""
    if center not in internal:
        return
    cycle = internal[center]
    for i in range(-len(cycle),len(cycle)-1):
        if cycle[i] in placements and cycle[i+1] not in placements:
            s,t = cycle[i],cycle[i+1]
            theta = acxyz(radii[center],radii[s],radii[t])
            offset = (placements[s]-placements[center])/(radii[s]+radii[center])
            offset *= e**(-1j*theta)
            placements[t] = placements[center] + offset*(radii[t]+radii[center])
            place(placements,radii,internal,t)

def testgrid(packing,k,z,r,q,g):
    """Build grid of test points around q with grid size g"""
    for i in (-2,-1,0,1,2):
        for j in (-2,-1,0,1,2):
            center = q + i*g + j*1j*g
            if abs(center-z) < r:
                newpack = InvertPacking(packing,center)
                newpack = NormalizePacking(newpack,k)
                minrad = min(r for z,r in newpack.values())
                yield minrad,i,j,center



"""TwoSatisfiability.py

Algorithms for solving 2-satisfiability problems.
For theory and references, see http://en.wikipedia.org/wiki/2-satisfiability

All instances should be represented as a directed implication graph in which
the vertices represent variables and (via Not.py) their negations. A variable
may be represented by any hashable Python object, and its negation should
be represented by the object Not(x). For instance, the implication graph
    {1:[2,3], 2:[Not(1),3]}
from the unit tests of this module represents the system of implications
among three logical variables v1, v2, and v3:
    v1 => v2, v1 => v3;  v2 => ~v1, v2 => v3.
An instance is satisfiable if it is possible to assign the Boolean values
True and False to these variables in order to make all implications become
logically correct. These problems have many applications involving problems
in which variables may take on either of two values and pairs of variables
are subject to arbitrary constraints; see the Wikipedia article for details.

If G is a graph of this type,
- Symmetrize(G) extends G by adding the contrapositive of each implication
- Satisfiable(G) returns True or False according to whether the
  2SAT instance can be satisfied. It takes linear time in the size of G.
- Forced(G) returns a dictionary mapping a subset of variables of G to
  values that they are forced to hold in any satisfying assignment.
  The empty dictionary is returned if all variables are free to take either
  truth value, and None is returned if the instance is unsatisfiable.
  Because this uses a reachability algorithm in directed acyclic graphs,
  it is not truly linear time but is still polynomial.

D. Eppstein, April 2009.
"""

import unittest
from Not import Not,SymbolicNegation
from Graphs import copyGraph
from StrongConnectivity import Condensation
from AcyclicReachability import Reachability

def Symmetrize(G):
    """Expand implication graph to a larger symmetric form.
    
    If the 2SAT instance includes an implication A=>B, then
    it is also valid to conclude that ~B => ~A, and our 2SAT solver
    needs to have that second implication made explicit.
    But we do not want to force users to supply the contrapositives
    for each of the implications they include, so we use this routine
    to fill in any missing implications.
    """
    H = copyGraph(G)
    for v in G:
        H.setdefault(Not(v),set())  # make sure all negations are included
        for w in G[v]:
            H.setdefault(w,set())   # as well as all implicants
            H.setdefault(Not(w),set()) # and negated implicants
    for v in G:
        for w in G[v]:
            H[Not(w)].add(Not(v))
    return H

def Satisfiable(G):
    """Does this 2SAT instance have a satisfying assignment?"""
    G = Condensation(Symmetrize(G))
    for C in G:
        for v in C:
            if Not(v) in C:
                return False
    return True

def Forced(G):
    """Find forced values for variables in a 2SAT instance.
    
    A variable's value is forced to x if every satisfying assignment
    assigns the same value x to that variable. We return a dictionary
    (possibly empty) in which the keys are the forced variables
    and their values are the values they are forced to.
    
    If the given instance is unsatisfiable, we return None."""
    Force = {}
    Sym = Symmetrize(G)
    Con = Condensation(Sym)
    Map = {}
    for SCC in Con:
        for v in SCC:
            Map[v] = SCC
    Reach = Reachability(Con)
    for v in Sym:
        if Reach.reachable(Map[v],Map[Not(v)]): # v implies not v?
            value = False
            if isinstance(v,SymbolicNegation):
                v = Not(v)
                value = True
            if v in Force:  # already added by negation?
                return None
            Force[v] = value
    return Force

# Unit tests
# Run python TwoSatisfiability.py to perform these tests.

class TwoSatTest(unittest.TestCase):
    T1 = {1:[2,3], 2:[Not(1),3]}
    T2 = {1:[2], 2:[Not(1)], Not(1):[3], 3:[4,2], 4:[1]}

    def testTwoSat(self):
        """Check that the correct problems are satisfiable."""
        self.assertEqual(Satisfiable(self.T1),True)
        self.assertEqual(Satisfiable(self.T2),False)

    def testForced(self):
        """Check that we can correctly identify forced variables."""
        self.assertEqual(Forced(self.T1),{1:False})
        self.assertEqual(Forced(self.T2),None)

if __name__ == "__main__":
    unittest.main()   



"""Eratosthenes.py
Space-efficient version of sieve of Eratosthenes.
D. Eppstein, May 2004.

The main storage of the algorithm is a hash table D with sqrt(n)
nonempty entries for a total of O(sqrt n) space.

At any point in the algorithm, each prime p occupies a cell with key at
most 2n.  E.g. by Bertrand's postulate, there is another prime p'
between n/p and 2n/p, and p' can not yet have been included because it
is greater than sqrt n, so key pp' can not be used by any other prime;
therefore p is placed at or before key pp'<2n.  Thus, the number of
times p can have been moved from its initial placement at p^2 is < n/p.
The time for the algorithm, up to output n, is O(n) + sum_{prime p <=
sqrt(n)} O(n/p) = O(n log log n).  The algorithm also makes a recursive
call, but the recursion only generates primes up to sqrt n so its time
and space is negligible compared to the outer call.

If efficiency is a significant concern it may be better to combine
this idea with segmentation and bitvectors, as in the code by
T. Oliveira e Silva at http://www.ieeta.pt/~tos/software/prime_sieve.html
Thanks to Alex Martelli for the suggestion of keeping one prime
per entry of D, rather than a list of all prime factors of D.

We also include a variant of the sieve that produces a list of all
integers, with their factorizations, and an application of this
variant in the generation of practical numbers.
"""

import unittest
from collections import defaultdict

def primes():
    '''Yields the sequence of primes via the Sieve of Eratosthenes.'''
    yield 2                 # Only even prime.  Sieve only odd numbers.

    # Generate recursively the sequence of primes up to sqrt(n).
    # Each p from the sequence is used to initiate sieving at p*p.
    roots = primes()
    root = roots.next()
    square = root*root

    # The main sieving loop.
    # We use a hash table D such that D[n]=2p for p a prime factor of n.
    # Each prime p up to sqrt(n) appears once as a value in D, and is
    # moved to successive odd multiples of p as the sieve progresses.
    D = {}
    n = 3
    while True:
        if n >= square:     # Time to include another square?
            D[square] = root+root
            root = roots.next()
            square = root*root

        if n not in D:      # Not witnessed, must be prime.
            yield n
        else:               # Move witness p to next free multiple.
            p = D[n]
            q = n+p
            while q in D:
                q += p
            del D[n]
            D[q] = p
        n += 2              # Move on to next odd number.

def FactoredIntegers():
    """
    Generate pairs n,F where F is the prime factorization of n.
    F is represented as a dictionary in which each prime factor of n
    is a key and the exponent of that prime is the corresponding value.
    """
    yield 1,{}
    i = 2
    factorization = defaultdict(dict)
    while True:
        if i not in factorization:  # prime
            F = {i:1}
            yield i,F
            factorization[2*i] = F
        elif len(factorization[i]) == 1:    # prime power
            p,x = factorization[i].items()[0]
            F = {p:x+1}
            yield i,F
            factorization[2*i] = F
            factorization[i+p**x][p] = x
            del factorization[i]
        else:
            yield i,factorization[i]
            for p,x in factorization[i].iteritems():
                q = p**x
                iq = i+q
                if iq in factorization and p in factorization[iq]:
                    iq += p**x  # skip higher power of p
                factorization[iq][p] = x
            del factorization[i]
        i += 1

def MoebiusSequence():
    """The sequence of values of the Moebius function, OEIS A008683."""
    for n,F in FactoredIntegers():
        if n > 1 and set(F.values()) != {1}:
            yield 0
        else:
            yield (-1)**len(F)

MoebiusFunctionValues = [None]
MoebiusFunctionIterator = MoebiusSequence()
def MoebiusFunction(n):
    """A functional version of the Moebius sequence.
    Efficient only for small values of n."""
    while n >= len(MoebiusFunctionValues):
        MoebiusFunctionValues.append(MoebiusFunctionIterator.next())
    return MoebiusFunctionValues[n]

def isPracticalFactorization(f):
    """Test whether f is the factorization of a practical number."""
    f = f.items()
    f.sort()
    sigma = 1
    for p,x in f:
        if sigma < p - 1:
            return False
        sigma *= (p**(x+1)-1)//(p-1)
    return True

def PracticalNumbers():
    """Generate the sequence of practical (or panarithmic) numbers."""
    for x,f in FactoredIntegers():
        if isPracticalFactorization(f):
            yield x

# If run standalone, perform unit tests
class SieveTest(unittest.TestCase):    
    def testPrime(self):
        """Test that the first few primes are generated correctly."""
        G = primes()
        for p in [2,3,5,7,11,13,17,19,23,29,31,37]:
            self.assertEqual(p,G.next())

    def testPractical(self):
        """Test that the first few practical nos are generated correctly."""
        G = PracticalNumbers()
        for p in [1,2,4,6,8,12,16,18,20,24,28,30,32,36]:
            self.assertEqual(p,G.next())

if __name__ == "__main__":
    unittest.main()



"""SMAWK.py

Totally monotone matrix searching algorithms.

The offline algorithm in ConcaveMinima is from Agarwal, Klawe, Moran,
Shor, and Wilbur, Geometric applications of a matrix searching algorithm,
Algorithmica 2, pp. 195-208 (1987).

The online algorithm in OnlineConcaveMinima is from Galil and Park,
A linear time algorithm for concave one-dimensional dynamic programming,
manuscript, 1989, which simplifies earlier work on the same problem
by Wilbur (J. Algorithms 1988) and Eppstein (J. Algorithms 1990).

D. Eppstein, March 2002, significantly revised August 2005
"""

def ConcaveMinima(RowIndices,ColIndices,Matrix):
    """
    Search for the minimum value in each column of a matrix.
    The return value is a dictionary mapping ColIndices to pairs
    (value,rowindex). We break ties in favor of earlier rows.
    
    The matrix is defined implicitly as a function, passed
    as the third argument to this routine, where Matrix(i,j)
    gives the matrix value at row index i and column index j.
    The matrix must be concave, that is, satisfy the property
        Matrix(i,j) > Matrix(i',j) => Matrix(i,j') > Matrix(i',j')
    for every i<i' and j<j'; that is, in every submatrix of
    the input matrix, the positions of the column minima
    must be monotonically nondecreasing.
    
    The rows and columns of the matrix are labeled by the indices
    given in order by the first two arguments. In most applications,
    these arguments can simply be integer ranges.
    """

    # Base case of recursion
    if not ColIndices: return {}
    
    # Reduce phase: make number of rows at most equal to number of cols
    stack = []
    for r in RowIndices:
        while len(stack) >= 1 and \
                Matrix(stack[-1], ColIndices[len(stack)-1]) \
                > Matrix(r, ColIndices[len(stack)-1]):
            stack.pop()
        if len(stack) != len(ColIndices):
            stack.append(r)
    RowIndices = stack
    
    # Recursive call to search for every odd column
    minima = ConcaveMinima(RowIndices,
                [ColIndices[i] for i in xrange(1,len(ColIndices),2)],
                Matrix)

    # Go back and fill in the even rows
    r = 0
    for c in xrange(0,len(ColIndices),2):
        col = ColIndices[c]
        row = RowIndices[r]
        if c == len(ColIndices) - 1:
            lastrow = RowIndices[-1]
        else:
            lastrow = minima[ColIndices[c+1]][1]
        pair = (Matrix(row,col),row)
        while row != lastrow:
            r += 1
            row = RowIndices[r]
            pair = min(pair,(Matrix(row,col),row))
        minima[col] = pair

    return minima

class OnlineConcaveMinima:
    """
    Online concave minimization algorithm of Galil and Park.
    
    OnlineConcaveMinima(Matrix,initial) creates a sequence of pairs
    (self.value(j),self.index(j)), where
        self.value(0) = initial,
        self.value(j) = min { Matrix(i,j) | i < j } for j > 0,
    and where self.index(j) is the value of j that provides the minimum.
    Matrix(i,j) must be concave, in the same sense as for ConcaveMinima.
    
    We never call Matrix(i,j) until value(i) has already been computed,
    so that the Matrix function may examine previously computed values.
    Calling value(i) for an i that has not yet been computed forces
    the sequence to be continued until the desired index is reached.
    Calling iter(self) produces a sequence of (value,index) pairs.
    
    Matrix(i,j) should always return a value, rather than raising an
    exception, even for j larger than the range we expect to compute.
    If j is out of range, a suitable value to return that will not
    violate concavity is Matrix(i,j) = -i.  It will not work correctly
    to return a flag value such as None for large j, because the ties
    formed by the equalities among such flags may violate concavity.
    """
    
    def __init__(self,Matrix,initial):
        """Initialize a OnlineConcaveMinima object."""

        # State used by self.value(), self.index(), and iter(self)
        self._values = [initial]    # tentative solution values...
        self._indices = [None]      # ...and their indices
        self._finished = 0          # index of last non-tentative value

        # State used by the internal algorithm
        #
        # We allow self._values to be nonempty for indices > finished,
        # keeping invariant that
        # (1) self._values[i] = Matrix(self._indices[i], i),
        # (2) if the eventual correct value of self.index(i) < base,
        #     then self._values[i] is nonempty and correct.
        #
        # In addition, we keep a column index self._tentative, such that
        # (3) if i <= tentative, and the eventual correct value of
        #     self.index(i) <= finished, then self._values[i] is correct.
        #
        self._matrix = Matrix
        self._base = 0
        self._tentative = 0

    def __iter__(self):
        """Loop through (value,index) pairs."""
        i = 0
        while True:
            yield self.value(i),self.index(i)
            i += 1

    def value(self,j):
        """Return min { Matrix(i,j) | i < j }."""
        while self._finished < j:
            self._advance()
        return self._values[j]

    def index(self,j):
        """Return argmin { Matrix(i,j) | i < j }."""
        while self._finished < j:
            self._advance()
        return self._indices[j]

    def _advance(self):
        """Finish another value,index pair."""
        # First case: we have already advanced past the previous tentative
        # value.  We make a new tentative value by applying ConcaveMinima
        # to the largest square submatrix that fits under the base.
        i = self._finished + 1
        if i > self._tentative:
            rows = range(self._base,self._finished+1)
            self._tentative = self._finished+len(rows)
            cols = range(self._finished+1,self._tentative+1)
            minima = ConcaveMinima(rows,cols,self._matrix)
            for col in cols:
                if col >= len(self._values):
                    self._values.append(minima[col][0])
                    self._indices.append(minima[col][1])
                elif minima[col][0] < self._values[col]:
                    self._values[col],self._indices[col] = minima[col]
            self._finished = i
            return
        
        # Second case: the new column minimum is on the diagonal.
        # All subsequent ones will be at least as low,
        # so we can clear out all our work from higher rows.
        # As in the fourth case, the loss of tentative is
        # amortized against the increase in base.
        diag = self._matrix(i-1,i)
        if diag < self._values[i]:
            self._values[i] = diag
            self._indices[i] = self._base = i-1
            self._tentative = self._finished = i
            return
        
        # Third case: row i-1 does not supply a column minimum in
        # any column up to tentative. We simply advance finished
        # while maintaining the invariant.
        if self._matrix(i-1,self._tentative) >= self._values[self._tentative]:
            self._finished = i
            return
        
        # Fourth and final case: a new column minimum at self._tentative.
        # This allows us to make progress by incorporating rows
        # prior to finished into the base.  The base invariant holds
        # because these rows cannot supply any later column minima.
        # The work done when we last advanced tentative (and undone by
        # this step) can be amortized against the increase in base.
        self._base = i-1
        self._tentative = self._finished = i
        return



"""SortedSet.py

Set data structure augmented by a method to list all items in the set in sorted order.

Two naive methods to do this would be
  (1) just use a Python set type and sort when requested, or
  (2) use a skip list or balanced binary search tree to store the items.
Our solution is almost as simple as (1) but has the same amortized
efficiency as (2): store a list of the items in the sorted order they
were most recently output as, together with small lists of the changes
to the set since then, and handle requests to list all items by
sorting the change lists and merging with the larger sorted list.

D. Eppstein, August 2008.
"""

from itertools import chain

class SortedSet:
    """Maintain a set of items in such a way that iter(set) returns the items in sorted order.
    We also allow a custom comparison routine, and augment the usual add() and remove() methods
    with an update() method that tells the set that a single item's position in the order might
    have changed."""
    
    def __init__(self,iterable=[],comparison=None):
        """Create a new sorted set with the given comparison function."""
        self._comparison = comparison
        self._set = set(iterable)
        self._previous = None

    def __len__(self):
        """How many items do we have?"""
        return len(self._set)

    def add(self,item):
        """Add the given item to a sorted set."""
        self._set.add(item)
        if self._previous:
            self._additions.add(item)

    def remove(self,item):
        """Remove the given item from a sorted set."""
        self._set.remove(item)
        if self._previous:
            self._removals.add(item)
            self._additions.discard(item)

    def update(self,item):
        """Flag the given item as needing re-comparison with its neighbors in the order."""
        if self._previous:
            self._removals.add(item)
            self._additions.add(item)

    def __iter__(self):
        if not self._previous:
            self._previous = sorted(self._set,cmp=self._comparison)
        else:
            self._previous = sorted(chain(self._additions,
                                          (x for x in self._previous if x not in self._removals)),
                                    cmp=self._comparison)
        self._removals = set()
        self._additions = set()
        return iter(self._previous)

# ======================================================================
#   Unit tests
# ======================================================================

if __name__ == "__main__":
    import unittest

    class SortedSetTest(unittest.TestCase):
        def testSortedSet(self):
            """Test whether SortedSet works correctly."""
            S = SortedSet()
            self.assertEqual(len(S),0)
            S.add(1)
            S.add(4)
            S.add(2)
            S.add(9)
            S.add(3)
            self.assertEqual(list(S),[1,2,3,4,9])
            self.assertEqual(len(S),5)
            S.remove(4)
            S.add(6)
            S.add(5)
            S.add(7)
            S.remove(6)
            S.remove(1)
            S.remove(2)
            S.add(1)
            self.assertEqual(list(S),[1,3,5,7,9])
            self.assertEqual(list(SortedSet([1,3,6,7])),[1,3,6,7])

    print "Testing SortedSet.py"
    unittest.main()   



"""Lyndon.py
Algorithms on strings and sequences based on Lyndon words.
David Eppstein, October 2011."""

import unittest
from Eratosthenes import MoebiusFunction

def LengthLimitedLyndonWords(s,n):
    """Generate nonempty Lyndon words of length <= n over an s-symbol alphabet.
    The words are generated in lexicographic order, using an algorithm from
    J.-P. Duval, Theor. Comput. Sci. 1988, doi:10.1016/0304-3975(88)90113-2.
    As shown by Berstel and Pocchiola, it takes constant average time
    per generated word."""
    w = [-1]                            # set up for first increment
    while w:
        w[-1] += 1                      # increment the last non-z symbol
        yield w
        m = len(w)
        while len(w) < n:               # repeat word to fill exactly n syms
            w.append(w[-m])
        while w and w[-1] == s - 1:     # delete trailing z's
            w.pop()

def LyndonWordsWithLength(s,n):
    """Generate Lyndon words of length exactly n over an s-symbol alphabet.
    Since nearly half of the outputs of LengthLimitedLyndonWords(s,n)
    have the desired length, it again takes constant average time per word."""
    if n == 0:
        yield []    # the empty word is a special case not handled by main alg
    for w in LengthLimitedLyndonWords(s,n):
        if len(w) == n:
            yield w

def LyndonWords(s):
    """Generate all Lyndon words over an s-symbol alphabet.
    The generation order is by length, then lexicographic within each length."""
    n = 0
    while True:
        for w in LyndonWordsWithLength(s,n):
            yield w
        n += 1

def DeBruijnSequence(s,n):
    """Generate a De Bruijn sequence for words of length n over s symbols
    by concatenating together in lexicographic order the Lyndon words
    whose lengths divide n. The output length will be s^n.
    Because nearly half of the generated sequences will have length
    exactly n, the algorithm will take O(s^n/n) steps, and the bulk
    of the time will be spent in sequence concatenation."""
    
    output = []
    for w in LengthLimitedLyndonWords(s,n):
        if n % len(w) == 0:
            output += w
    return output

def CountLyndonWords(s,n):
    """The number of length-n Lyndon words over s symbols."""
    if n == 0:
        return 1
    total = 0
    for i in range(1,n+1):
        if n%i == 0:
            total += MoebiusFunction(n/i) * s**i
    return total//n

def ChenFoxLyndonBreakpoints(s):
    """Find starting positions of Chen-Fox-Lyndon decomposition of s.
    The decomposition is a set of Lyndon words that start at 0 and
    continue until the next position. 0 itself is not output, but
    the final breakpoint at the end of s is. The argument s must be
    of a type that can be indexed (e.g. a list, tuple, or string).
    The algorithm follows Duval, J. Algorithms 1983, but uses 0-based
    indexing rather than Duval's choice of 1-based indexing."""
    k = 0
    while k < len(s):
        i,j = k,k+1
        while j < len(s) and s[i] <= s[j]:
            i = (s[i] == s[j]) and i+1 or k     # Python cond?yes:no syntax
            j += 1
        while k < i+1:
            k += j-i
            yield k

def ChenFoxLyndon(s):
    """Decompose s into Lyndon words according to the Chen-Fox-Lyndon theorem.
    The arguments are the same as for ChenFoxLyndonBreakpoints but the
    return values are subsequences of s rather than indices of breakpoints."""
    old = 0
    for k in ChenFoxLyndonBreakpoints(s):
        yield s[old:k]
        old = k

def SmallestSuffix(s):
    """Find the suffix of s that is smallest in lexicographic order."""
    for w in ChenFoxLyndon(s):
        pass
    return w

def SmallestRotation(s):
    """Find the rotation of s that is smallest in lexicographic order.
    Duval 1983 describes how to modify his algorithm to do so but I think
    it's cleaner and more general to work from the ChenFoxLyndon output."""
    prev,rep = None,0
    for w in ChenFoxLyndon(s+s):
        if w == prev:
            rep += 1
        else:
            prev,rep = w,1
        if len(w)*rep == len(s):
            return w*rep
    raise Exception("Reached end of factorization with no shortest rotation")

def isLyndonWord(s):
    """Is the given sequence a Lyndon word?"""
    if len(s) == 0:
        return True
    return ChenFoxLyndonBreakpoints(s).next() == len(s)

# If run standalone, perform unit tests
class LyndonTest(unittest.TestCase):
    def testCount(self):
        """Test that we count Lyndon words correctly."""
        for s in range(2,7):
            for n in range(1,6):
                self.assertEqual(CountLyndonWords(s,n),
                    len(list(LyndonWordsWithLength(s,n))))

    def testOrder(self):
        """Test that we generate Lyndon words in lexicographic order."""
        for s in range(2,7):
            for n in range(1,6):
                prev = []
                for x in LengthLimitedLyndonWords(s,n):
                    self.assert_(prev < x)
                    prev = list(x)

    def testSubsequence(self):
        """Test that words of length n-1 are a subsequence of length n."""
        for s in range(2,7):
            for n in range(2,6):
                smaller = LengthLimitedLyndonWords(s,n-1)
                for x in LengthLimitedLyndonWords(s,n):
                    if len(x) < n:
                        self.assertEqual(x,smaller.next())
    
    def testIsLyndon(self):
        """Test that the words we generate are Lyndon words."""
        for s in range(2,7):
            for n in range(2,6):
                for w in LengthLimitedLyndonWords(s,n):
                    self.assertEqual(isLyndonWord(w), True)
    
    def testNotLyndon(self):
        """Test that words that are not Lyndon words aren't claimed to be."""
        nl = sum(1 for i in range(8**4) if isLyndonWord("%04o" % i))
        self.assertEqual(nl,CountLyndonWords(8,4))

    def testDeBruijn(self):
        """Test that the De Bruijn sequence is correct."""
        for s in range(2,7):
            for n in range(1,6):
                db = DeBruijnSequence(s,n)
                self.assertEqual(len(db), s**n)
                db = db + db    # duplicate so we can wrap easier
                subs = set(tuple(db[i:i+n]) for i in range(s**n))
                self.assertEqual(len(subs), s**n)

if __name__ == "__main__":
    unittest.main()



"""CardinalityMatching.py

Find maximum cardinality matchings in general undirected graphs.

D. Eppstein, UC Irvine, September 6, 2003.
"""

import sys

from UnionFind import UnionFind
from Util import arbitrary_item

def matching(G, initialMatching = None):
    """Find a maximum cardinality matching in a graph G.
    G is represented in modified GvR form: iter(G) lists its vertices;
    iter(G[v]) lists the neighbors of v; w in G[v] tests adjacency.
    For maximal efficiency, G and G[v] should be dictionaries, so
    that adjacency tests take constant time each.
    The output is a dictionary mapping vertices to their matches;
    unmatched vertices are omitted from the dictionary.

    We use Edmonds' blossom-contraction algorithm, as described e.g.
    in Galil's 1986 Computing Surveys paper.
    """

    # Copy initial matching so we can use it nondestructively
    # and augment it greedily to reduce main loop iterations
    matching = greedyMatching(G,initialMatching)

    def augment():
        """Search for a single augmenting path.
        Returns true if the matching size was increased, false otherwise.
        """

        # Data structures for augmenting path search:
        #
        # leader: union-find structure; the leader of a blossom is one
        # of its vertices (not necessarily topmost), and leader[v] always
        # points to the leader of the largest blossom containing v
        #
        # S: dictionary of blossoms at even levels of the structure tree.
        # Dictionary keys are names of blossoms (as returned by the union-find
        # data structure) and values are the structure tree parent of the blossom
        # (a T-node, or the top vertex if the blossom is a root of a structure tree).
        #
        # T: dictionary of vertices at odd levels of the structure tree.
        # Dictionary keys are the vertices; T[x] is a vertex with an unmatched
        # edge to x.  To find the parent in the structure tree, use leader[T[x]].
        #
        # unexplored: collection of unexplored vertices within blossoms of S
        #
        # base: if x was originally a T-vertex, but becomes part of a blossom,
        # base[t] will be the pair (v,w) at the base of the blossom, where v and t
        # are on the same side of the blossom and w is on the other side.

        leader = UnionFind()
        S = {}
        T = {}
        unexplored = []
        base = {}

        # Subroutines for augmenting path search.
        # Many of these are called only from one place, but are split out
        # as subroutines to improve modularization and readability.

        def blossom(v,w,a):
            """Create a new blossom from edge v-w with common ancestor a."""

            def findSide(v,w):
                path = [leader[v]]
                b = (v,w)   # new base for all T nodes found on the path
                while path[-1] != a:
                    tnode = S[path[-1]]
                    path.append(tnode)
                    base[tnode] = b
                    unexplored.append(tnode)
                    path.append(leader[T[tnode]])
                return path

            a = leader[a]   # sanity check
            path1,path2 = findSide(v,w), findSide(w,v)
            leader.union(*path1)
            leader.union(*path2)
            S[leader[a]] = S[a] # update structure tree

        topless = object()  # should be unequal to any graph vertex
        def alternatingPath(start, goal = topless):
            """Return sequence of vertices on alternating path from start to goal.
            The goal must be a T node along the path from the start to
            the root of the structure tree. If goal is omitted, we find
            an alternating path to the structure tree root.
            """
            path = []
            while 1:
                while start in T:
                    v, w = base[start]
                    vs = alternatingPath(v, start)
                    vs.reverse()
                    path += vs
                    start = w
                path.append(start)
                if start not in matching:
                    return path     # reached top of structure tree, done!
                tnode = matching[start]
                path.append(tnode)
                if tnode == goal:
                    return path     # finished recursive subpath
                start = T[tnode]

        def alternate(v):
            """Make v unmatched by alternating the path to the root of its structure tree."""
            path = alternatingPath(v)
            path.reverse()
            for i in range(0,len(path)-1,2):
                matching[path[i]] = path[i+1]
                matching[path[i+1]] = path[i]

        def addMatch(v, w):
            """Here with an S-S edge vw connecting vertices in different structure trees.
            Find the corresponding augmenting path and use it to augment the matching.
            """
            alternate(v)
            alternate(w)
            matching[v] = w
            matching[w] = v

        def ss(v,w):
            """Handle detection of an S-S edge in augmenting path search.
            Like augment(), returns true iff the matching size was increased.
            """

            if leader[v] == leader[w]:
                return False        # self-loop within blossom, ignore

            # parallel search up two branches of structure tree
            # until we find a common ancestor of v and w
            path1, head1 = {}, v
            path2, head2 = {}, w

            def step(path, head):
                head = leader[head]
                parent = leader[S[head]]
                if parent == head:
                    return head     # found root of structure tree
                path[head] = parent
                path[parent] = leader[T[parent]]
                return path[parent]

            while 1:
                head1 = step(path1, head1)
                head2 = step(path2, head2)

                if head1 == head2:
                    blossom(v, w, head1)
                    return False

                if leader[S[head1]] == head1 and leader[S[head2]] == head2:
                    addMatch(v, w)
                    return True

                if head1 in path2:
                    blossom(v, w, head1)
                    return False

                if head2 in path1:
                    blossom(v, w, head2)
                    return False

        # Start of main augmenting path search code.

        for v in G:
            if v not in matching:
                S[v] = v
                unexplored.append(v)

        current = 0     # index into unexplored, in FIFO order so we get short paths
        while current < len(unexplored):
            v = unexplored[current]
            current += 1

            for w in G[v]:
                if leader[w] in S:  # S-S edge: blossom or augmenting path
                    if ss(v,w):
                        return True

                elif w not in T:    # previously unexplored node, add as T-node
                    T[w] = v
                    u = matching[w]
                    if leader[u] not in S:
                        S[u] = w    # and add its match as an S-node
                        unexplored.append(u)

        return False    # ran out of graph without finding an augmenting path

    # augment the matching until it is maximum
    while augment():
        pass

    return matching

def greedyMatching(G, initialMatching=None):
    """Near-linear-time greedy heuristic for creating high-cardinality matching.
    If there is any vertex with one unmatched neighbor, we match it.
    Otherwise, if there is a vertex with two unmatched neighbors, we contract
    it away and store the contraction on a stack for later matching.
    If neither of these two cases applies, we match an arbitrary edge.
    """

    # Copy initial matching so we can use it nondestructively
    matching = {}
    if initialMatching:
        for x in initialMatching:
            matching[x] = initialMatching[x]

    # Copy graph to new subgraph of available edges
    # Representation: nested dictionary rep->rep->pair
    # where the reps are representative vertices for merged clusters
    # and the pair is an unmatched original pair of vertices
    avail = {}
    has_edge = False
    for v in G:
        if v not in matching:
            avail[v] = {}
            for w in G[v]:
                if w not in matching:
                    avail[v][w] = (v,w)
                    has_edge = True
            if not avail[v]:
                del avail[v]
    if not has_edge:
        return matching

    # make sets of degree one and degree two vertices
    deg1 = {v for v in avail if len(avail[v]) == 1}
    deg2 = {v for v in avail if len(avail[v]) == 2}
    d2edges = []
    def updateDegree(v):
        """Cluster degree changed, update sets."""
        if v in deg1:
            deg1.remove(v)
        elif v in deg2:
            deg2.remove(v)
        if len(avail[v]) == 0:
            del avail[v]
        elif len(avail[v]) == 1:
            deg1.add(v)
        elif len(avail[v]) == 2:
            deg2.add(v)

    def addMatch(v,w):
        """Add edge connecting two given cluster reps, update avail."""
        p,q = avail[v][w]
        matching[p] = q
        matching[q] = p
        for x in avail[v].keys():
            if x != w:
                del avail[x][v]
                updateDegree(x)
        for x in avail[w].keys():
            if x != v:
                del avail[x][w]
                updateDegree(x)
        avail[v] = avail[w] = {}
        updateDegree(v)
        updateDegree(w)

    def contract(v):
        """Handle degree two vertex."""
        u,w = avail[v]  # find reps for two neighbors
        d2edges.extend([avail[v][u],avail[v][w]])
        del avail[u][v]
        del avail[w][v]
        if len(avail[u]) > len(avail[w]):
            u,w = w,u   # swap to preserve near-linear time bound
        for x in avail[u].keys():
            del avail[x][u]
            if x in avail[w]:
                updateDegree(x)
            elif x != w:
                avail[x][w] = avail[w][x] = avail[u][x]
        avail[u] = avail[v] = {}
        updateDegree(u)
        updateDegree(v)
        updateDegree(w)

    # loop adding edges or contracting deg2 clusters
    while avail:
        if deg1:
            v = arbitrary_item(deg1)
            w = arbitrary_item(avail[v])
            addMatch(v,w)
        elif deg2:
            v = arbitrary_item(deg2)
            contract(v)
        else:
            v = arbitrary_item(avail)
            w = arbitrary_item(avail[v])
            addMatch(v,w)

    # at this point the edges listed in d2edges form a matchable tree
    # repeat the degree one part of the algorithm only on those edges
    avail = {}
    d2edges = [(u,v) for u,v in d2edges if u not in matching and v not in matching]
    for u,v in d2edges:
        avail[u] = {}
        avail[v] = {}
    for u,v in d2edges:
        avail[u][v] = avail[v][u] = (u,v)
    deg1 = {v for v in avail if len(avail[v]) == 1}
    while deg1:
        v = arbitrary_item(deg1)
        w = arbitrary_item(avail[v])
        addMatch(v,w)

    return matching



"""PartialCube.py

Test whether a graph is an isometric subgraph of a hypercube.

D. Eppstein, September 2005, rewritten May 2007 per arxiv:0705.1025.
"""

import BFS
import Medium
from Bipartite import isBipartite
from UnionFind import UnionFind
from StrongConnectivity import StronglyConnectedComponents
from Graphs import isUndirected
import unittest

def PartialCubeEdgeLabeling(G):
    """
    Label edges of G by their equivalence classes in a partial cube structure.

    We follow the algorithm of arxiv:0705.1025, in which a number of
    equivalence classes equal to the maximum degree of G can be found
    simultaneously by a single breadth first search, using bitvectors.
    However, in order to avoid deep recursions (problematic in Python)
    we use a union-find data structure to keep track of edge identifications
    discovered so far. That is, we repeatedly contract our initial graph,
    maintaining as we do the property that G[v][w] points to a union-find
    set representing edges in the original graph that have been contracted
    to the single edge v-w.
    """
    
    # Some simple sanity checks
    if not isUndirected(G):
        raise Medium.MediumError("graph is not undirected")
    L = list(StronglyConnectedComponents(G))
    if len(L) != 1:
        raise Medium.MediumError("graph is not connected")

    # Set up data structures for algorithm:
    # - UF: union find data structure representing known edge equivalences
    # - CG: contracted graph at current stage of algorithm
    # - LL: limit on number of remaining available labels
    UF = UnionFind()
    CG = {v:{w:(v,w) for w in G[v]} for v in G}
    NL = len(CG)-1
    
    # Initial sanity check: are there few enough edges?
    # Needed so that we don't try to use union-find on a dense
    # graph and incur superquadratic runtimes.
    n = len(CG)
    m = sum([len(CG[v]) for v in CG])
    if 1<<(m//n) > n:
        raise Medium.MediumError("graph has too many edges")

    # Main contraction loop in place of the original algorithm's recursion
    while len(CG) > 1:    
        if not isBipartite(CG):
            raise Medium.MediumError("graph is not bipartite")

        # Find max degree vertex in G, and update label limit
        deg,root = max([(len(CG[v]),v) for v in CG])
        if deg > NL:
            raise Medium.MediumError("graph has too many equivalence classes")
        NL -= deg

        # Set up bitvectors on vertices
        bitvec = {v:0 for v in CG}
        neighbors = {}
        i = 0
        for neighbor in CG[root]:
            bitvec[neighbor] = 1<<i
            neighbors[1<<i] = neighbor
            i += 1

        # Breadth first search to propagate bitvectors to the rest of the graph
        for LG in BFS.BreadthFirstLevels(CG,root):
            for v in LG:
                for w in LG[v]:
                    bitvec[w] |= bitvec[v]

        # Make graph of labeled edges and union them together
        labeled = {v:set() for v in CG}
        for v in CG:
            for w in CG[v]:
                diff = bitvec[v]^bitvec[w]
                if not diff or bitvec[w] &~ bitvec[v] == 0:
                    continue    # zero edge or wrong direction
                if diff not in neighbors:
                    raise Medium.MediumError("multiply-labeled edge")
                neighbor = neighbors[diff]
                UF.union(CG[v][w],CG[root][neighbor])
                UF.union(CG[w][v],CG[neighbor][root])
                labeled[v].add(w)
                labeled[w].add(v)

        # Map vertices to components of labeled-edge graph
        component = {}
        compnum = 0
        for SCC in StronglyConnectedComponents(labeled):
            for v in SCC:
                component[v] = compnum
            compnum += 1

        # generate new compressed subgraph
        NG = {i:{} for i in range(compnum)}
        for v in CG:
            for w in CG[v]:
                if bitvec[v] == bitvec[w]:
                    vi = component[v]
                    wi = component[w]
                    if vi == wi:
                        raise Medium.MediumError("self-loop in contracted graph")
                    if wi in NG[vi]:
                        UF.union(NG[vi][wi],CG[v][w])
                    else:
                        NG[vi][wi] = CG[v][w]
        
        CG = NG

    # Here with all edge equivalence classes represented by UF.
    # Turn them into a labeled graph and return it.
    return {v:{w:UF[v,w] for w in G[v]} for v in G}


def MediumForPartialCube(G):
    """
    Find a medium corresponding to the partial cube G.
    Raises MediumError if G is not a partial cube.
    Uses the O(n^2) time algorithm of arxiv:0705.1025.
    """
    L = PartialCubeEdgeLabeling(G)
    M = Medium.LabeledGraphMedium(L)
    Medium.RoutingTable(M)   # verification step per arxiv:0705.1025
    return M


def PartialCubeLabeling(G):
    """Return vertex labels with Hamming distance = graph distance."""
    return Medium.HypercubeEmbedding(MediumForPartialCube(G))


def isPartialCube(G):
    """Test whether the given graph is a partial cube."""
    try:
        MediumForPartialCube(G)
        return True
    except Medium.MediumError:
        return False



# Perform some sanity checks if run standalone

class PartialCubeTest(unittest.TestCase):

    # make medium from all five-bit numbers that have 2 or 3 bits lit
    twobits = [3,5,6,9,10,12,17,18,20,24]
    threebits = [31^x for x in twobits]
    M523 = Medium.BitvectorMedium(twobits+threebits,5)

    def testIsPartialCube(self):
        M = PartialCubeTest.M523
        G = Medium.StateTransitionGraph(M)
        I = isPartialCube(G)
        self.assertEqual(I,True)
    
    def testK4(self):
        G = {i:[j for j in range(4) if j != i] for i in range(4)}
        self.assertEqual(isPartialCube(G),False)

    def testK33(self):
        G = {0:[3,4,5],1:[3,4,5],2:[3,4,5],3:[0,1,2],4:[0,1,2],5:[0,1,2]}
        self.assertEqual(isPartialCube(G),False)

    def testMediumForPartialCube(self):
        """Check that we get an isomorphic medium via MediumForPartialCube."""
        # Note that we do not get the same tokens.
        # So, we need to check equality of graphs
        # rather than equality of media.
        M = PartialCubeTest.M523
        G = Medium.StateTransitionGraph(M)
        E = MediumForPartialCube(G)
        H = Medium.StateTransitionGraph(E)
        self.assertEqual(set(G),set(H))
        for v in G:
            self.assertEqual(set(G[v]),set(H[v]))

    def test6212(self):
        """
        A graph that can be labeled, but is not a partial cube.
        Tests the code in LabeledGraphMedium that checks for the
        existence of multiple equally labeled edges at a vertex.
        """
        n,a,b,c = 6,2,1,2
        G = {}
        for i in range(n):
            G[i,0] = [(i,1),((i+b)%n,3),((i+c)%n,3)]
            G[i,1] = [(i,0),(i,2),((i-a)%n,2)]
            G[i,2] = [(i,1),(i,3),((i+a)%n,1)]
            G[i,3] = [(i,2),((i-b)%n,0),((i-c)%n,0)]
        self.assertEqual(isPartialCube(G),False)

    def test61150(self):
        """
        Another graph that can be labeled, but is not a partial cube.
        Tests the code in RoutingTable that makes sure that only tokens
        in a single direction belong to the initial list of active tokens.
        """
        G = {}
        n,a,b,c,d = 6,1,1,5,0
        for i in range(n):
            G[i,0] = [(i,1),((i+a)%n,5),((i+b)%n,3)]
            G[i,1] = [(i,0),(i,2),((i-d)%n,4)]
            G[i,2] = [(i,1),(i,3),((i+c)%n,5)]
            G[i,3] = [(i,2),(i,4),((i-b)%n,0)]
            G[i,4] = [(i,3),(i,5),((i+d)%n,1)]
            G[i,5] = [(i,4),((i-a)%n,0),((i-c)%n,2)]
        self.assertEqual(isPartialCube(G),False)

if __name__ == "__main__":
    unittest.main()



"""Permutations.py
Efficient recursive version of the Steinhaus-Johnson-Trotter algorithm
for listing all permutations of a set of items.
D. Eppstein, October 2011.

NOTE: The generators in this module work by making a sequence of
small changes to a permutation stored in a Python list object,
and then yielding that list. That means that the objects that they
generate (the list objects) must not be modified by their callers,
because that would break subsequent generation steps. And it also
means that each permutation yielded by the generator only has its
correct value up to the time the generator is called again, after
which this value will change. If you want to keep a persistent copy
of a permutation, or change it, you need to copy it into a
separate object. E.g.

    RIGHT:
        [list(p) for p in SteinhausJohnsonTrotter(n)]
        # makes a list of all the permutations of order n
    WRONG:
        list(SteinhausJohnsonTrotter(n))
        [p for p in SteinhausJohnsonTrotter(n)]
        # either way, these make a list of length n! all of whose
        # elements point to the same list object as each other

The Steinhaus-Johnson-Trotter implementation given here sets up a
sequence of recursive simple generators, each taking constant space,
for a total space of O(n), where n is the number of items being permuted.
The number of recursive calls to generate a swap that moves the item
originally in position n of the input permutation is O(n-i+1), so all
but a 1/n fraction of the swaps take no recursion and the rest always take O(n) time, for an average time per swap of O(1) and an average time per
generated permutation of O(1). The other generators are similar.
"""

import unittest

def PlainChanges(n):
    """Generate the swaps for the Steinhaus-Johnson-Trotter algorithm."""
    if n < 1:
        return
    up = xrange(n-1)
    down = xrange(n-2,-1,-1)
    recur = PlainChanges(n-1)
    try:
        while True:
            for x in down:
                yield x
            yield recur.next() + 1
            for x in up:
                yield x
            yield recur.next()
    except StopIteration:
        pass

def SteinhausJohnsonTrotter(x):
    """Generate all permutations of x.
    If x is a number rather than an iterable, we generate the permutations
    of range(x)."""

    # set up the permutation and its length
    try:
        perm = list(x)
    except:
        perm = range(x)
    n = len(perm)

    # run through the sequence of swaps
    yield perm
    for x in PlainChanges(n):
        perm[x],perm[x+1] = perm[x+1],perm[x]
        yield perm

def DoublePlainChanges(n):
    """Generate the swaps for double permutations."""
    if n < 1:
        return
    up = xrange(1,2*n-1)
    down = xrange(2*n-2,0,-1)
    recur = DoublePlainChanges(n-1)
    try:
        while True:
            for x in up:
                yield x
            yield recur.next() + 1
            for x in down:
                yield x
            yield recur.next() + 2
    except StopIteration:
        pass

def DoubleSteinhausJohnsonTrotter(n):
    """Generate all double permutations of the range 0 through n-1"""
    perm = []
    for i in range(n):
        perm += [i,i]

    # run through the sequence of swaps
    yield perm
    for x in DoublePlainChanges(n):
        perm[x],perm[x+1] = perm[x+1],perm[x]
        yield perm

def StirlingChanges(n):
    """Variant Steinhaus-Johnson-Trotter for Stirling permutations.
    A Stirling permutation is a double permutation in which each
    pair of values has only larger values between them.
    The algorithm is to sweep the largest pair of values through
    the sequence of smaller values, recursing when it reaches
    the ends of the sequence, exactly as in the standard
    Steinhaus-Johnson-Trotter algorithm. However, it differs
    in swapping items two positions apart instead of adjacent items."""
    if n <= 1:
        return
    up = xrange(2*n-2)
    down = xrange(2*n-3,-1,-1)
    recur = StirlingChanges(n-1)
    try:
        while True:
            for x in down:
                yield x
            yield recur.next() + 2
            for x in up:
                yield x
            yield recur.next()
    except StopIteration:
        pass

def StirlingPermutations(n):
    """Generate all Stirling permutations of order n."""
    perm = []
    for i in range(n):
        perm += [i,i]

    # run through the sequence of swaps
    yield perm
    for x in StirlingChanges(n):
        perm[x],perm[x+2] = perm[x+2],perm[x]
        yield perm

def InvolutionChanges(n):
    """Generate change sequence for involutions on n items.
    Uses a variation of the Steinhaus-Johnson-Trotter idea,
    in which we first recurse for n-1, generating involutions
    in which the last item is fixed, and then we the match
    for the last item back and forth over a recursively
    generated sequence for n-2."""
    if n <= 3:
        for c in [[],[],[0],[0,1,0]][n]:
            yield c
        return
    for c in InvolutionChanges(n-1):
        yield c
    yield n-2
    for i in range(n-4,-1,-1):
        yield i
    ic = InvolutionChanges(n-2)
    up = range(0,n-2)
    down = range(n-3,-1,-1)
    try:
        while True:
            yield ic.next() + 1
            for i in up:
                yield i
            yield ic.next()
            for i in down:
                yield i
    except StopIteration:
        yield n-4

def Involutions(n):
    """Generate involutions on n items.
    The first involution is always the one in which all items
    are mapped to themselves, and the last involution is the one
    in which only the final two items are swapped.
    Each two involutions differ by a change that either adds or
    removes an adjacent pair of swapped items, moves a swap target
    by one, or swaps two adjacent swap targets."""
    p = range(n)
    yield p
    for c in InvolutionChanges(n):
        x,y = p[c],p[c+1]   # current partners of c and c+1
        if x == c and y != c+1: x = c+1
        if x != c and y == c+1: y = c
        p[x],p[y],p[c],p[c+1] = c+1, c, y, x    # swap partners
        yield p

# If run standalone, perform unit tests
class PermutationTest(unittest.TestCase):    
    def testChanges(self):
        """Do we get the expected sequence of changes for n=3?"""
        self.assertEqual(list(PlainChanges(3)),[1,0,1,0,1])
    
    def testLengths(self):
        """Are the lengths of the generated sequences factorial?"""
        f = 1
        for i in range(2,7):
            f *= i
            self.assertEqual(f,len(list(SteinhausJohnsonTrotter(i))))
    
    def testDistinct(self):
        """Are all permutations in the sequence different from each other?"""
        for i in range(2,7):
            s = set()
            n = 0
            for x in SteinhausJohnsonTrotter(i):
                s.add(tuple(x))
                n += 1
            self.assertEqual(len(s),n)
    
    def testAdjacent(self):
        """Do consecutive permutations in the sequence differ by a swap?"""
        for i in range(2,7):
            last = None
            for p in SteinhausJohnsonTrotter(i):
                if last:
                    diffs = [j for j in range(i) if p[j] != last[j]]
                    self.assertEqual(len(diffs),2)
                    self.assertEqual(p[diffs[0]],last[diffs[1]])
                    self.assertEqual(p[diffs[1]],last[diffs[0]])
                last = list(p)
    
    def testListInput(self):
        """If given a list as input, is it the first output?"""
        for L in ([1,3,5,7], list('zyx'), [], [[]], range(20)):
            self.assertEqual(L,SteinhausJohnsonTrotter(L).next())

    def testInvolutions(self):
        """Are these involutions and do we have the right number of them?"""
        telephone = [1,1,2,4,10,26,76,232,764]
        for n in range(len(telephone)):
            count = 0
            sorted = range(n)
            invs = set()
            for p in Involutions(n):
                self.assertEqual([p[i] for i in p],sorted)
                invs.add(tuple(p))
                count += 1
            self.assertEqual(len(invs),count)
            self.assertEqual(len(invs),telephone[n])

if __name__ == "__main__":
    unittest.main()



"""Graphs.py

Various simple functions for graph input.

Each function's input graph G should be represented in such a way that "for v in G" loops through the vertices, and "G[v]" produces a list of the neighbors of v; for instance, G may be a dictionary mapping each vertex to its neighbor set.

D. Eppstein, April 2004.
"""

def isUndirected(G):
    """Check that G represents a simple undirected graph."""
    for v in G:
        if v in G[v]:
            return False
        for w in G[v]:
            if v not in G[w]:
                return False
    return True

def maxDegree(G):
    """Return the maximum vertex (out)degree of graph G."""
    return max([len(G[v]) for v in G])

def minDegree(G):
    """Return the minimum vertex (out)degree of graph G."""
    return min([len(G[v]) for v in G])

def copyGraph(G,adjacency_list_type=set):
    """
    Make a copy of a graph G and return the copy.
    Any information stored in edges G[v][w] is discarded.
    
    Most of the time, copy.deepcopy will be preferable to this function;
    however, unlike deepcopy, this function can change the data type
    of the adjacency list of the given graph.

    The second argument should be a callable that turns a sequence
    of neighbors into an appropriate representation of the adjacency list.
    Note that, while Set, list, and tuple are appropriate values for
    adjacency_list_type, dict is not -- use Util.map_to_constant instead.
    """
    return {v:adjacency_list_type(iter(G[v])) for v in G}

def InducedSubgraph(V,G,adjacency_list_type=set):
    """
    The subgraph consisting of all edges between pairs of vertices in V.
    """
    return {x:adjacency_list_type(y for y in G[x] if y in V)
            for x in G if x in V}

def union(*graphs):
    """Return a graph having all edges from the argument graphs."""
    out = {}
    for G in graphs:
        for v in G:
            out.setdefault(v,set()).update(list(G[v]))
    return out

def isIndependentSet(V,G):
    """
    True if V is an independent set of vertices in G, False otherwise.
    """
    class NonIndependent(Exception):
        pass

    def TestIndependent(seq):
        for x in seq:
            raise NonIndependent

    try:
        InducedSubgraph(V,G,TestIndependent)
        return True
    except NonIndependent:
        return False



"""Medium.py

Algorithms for media; see e.g. my paper arxiv:cs.DS/0206033.
As a very brief introduction to media theory:

- A medium consists of a set of states, a set of tokens,
  and an "action" that produces a new state St from a state S and a token t.
  An action is "effective" if St != S.
- Each token t has a "reverse" r such that St = V != S iff Vr = S != V.
- Any two states can be connected by a sequence of actions that
  uses each token at most once and does not use both a token and its reverse.
- In any sequence of effective actions taking a state back to itself, 
  the tokens can be matched up in token-reverse pairs.

The resulting theory is equivalent to that for partial cubes (graphs that
can be embedded in a distance-preserving way into hypercubes): the state-
transition graphs of media are partial cubes, and any partial cube
corresponds to a medium in this way. However, media theory provides a
different perspective on these structures that more closely resembles the
theory of finite automata.

D. Eppstein, May 2007.
"""
    
import BFS,DFS
from Graphs import isUndirected
import unittest

class MediumError(ValueError): pass

class Medium:
    """
    Base class for media.
    
    A medium is defined by four instance methods:
    - M.states() lists the states of M
    - M.tokens() lists the tokens of M
    - M.reverse(token) finds the token with opposite action to its argument
    - M.action(state,token) gives the result of applying that token
    These should be defined in subclasses; the base does not define them.
    
    In addition, we define methods (that may possibly be overridden):
    - iter(M) is a synonym for M.states()
    - len(M) is a synonym for len(M.states())
    - M.effective(state) lists tokens effective on that state
    - M[state] returns a dict mapping tokens to actions
    - M(state,token) is a synonym for M.action(state,token)
    """

    def __iter__(self):
        """Generate sequence of medium states."""
        return self.states()
    
    def __len__(self):
        """Return number of states in the medium."""
        i = 0
        for S in self.states():
            i += 1
        return i

    def __getitem__(self,S):
        """Construct dict mapping tokens to actions from state S."""
        return {t:self.action(S,t) for t in self.tokens()}

    def __call__(self,S,t):
        """Apply token t to state S."""
        return self.action(S,t)


class ExplicitMedium(Medium):
    """
    Medium in which all dicts M[state] have been precomputed.
    This can be less space-efficient than other representations
    (since it essentially involves a big table with size
    (# states) x (# tokens) but it makes all operations fast.
    """

    def __init__(self,M):
        """Form ExplicitMedium from any other kind of medium."""
        self._reverse = {t:M.reverse(t) for t in M.tokens()}
        self._action = {S:M[S] for S in M}

    # Basic classes needed to define any medium

    def states(self):
        return iter(self._action)

    def tokens(self):
        return iter(self._reverse)

    def reverse(self,t):
        return self._reverse[t]

    def action(self,S,t):
        return self._action[S][t]

    # Faster implementation of other medium functions

    def __len__(self):
        return len(self._action)

    def __getitem__(self,S):
        return self._action[S]


class BitvectorMedium(Medium):
    """
    Medium defined by a set of bitvectors.
    
    The tokens of the medium are pairs (i,b) where i is an index
    into a bitvector and b is a bit; the action of a token on a bitvector
    is to change the i'th bit to b, if the result is part of the set,
    and if not to leave the bitvector unchanged.
    
    We assume but do not verify that the bitvectors do form a medium;
    that is, that one can transform any bitvector in the set into any
    other via a sequence of actions of length equal to the Hamming
    distance between the vectors.
    """

    def __init__(self,states,L):
        """Initialize medium for set states and bitvector length L."""
        self._states = set(states)
        self._veclen = L
    
    def states(self):
        return iter(self._states)

    def tokens(self):
        for i in range(self._veclen):
            yield i,False
            yield i,True

    def reverse(self,t):
        i,b = t
        return i,not b

    def action(self,S,t):
        """
        Compute the action of token t on state S.
        We form the bitvector V that should correspond to St,
        then test whether V belongs to the given set of states.
        If so, we return it; otherwise, we return S itself.
        """
        i,b = t
        mask = 1<<i
        if b:
            V = S | mask
        else:
            V = S &~ mask
        if V in self._states:
            return V
        else:
            return S


def StateTransitionGraph(M):
    """
    Build a graph G describing the states and transitions of medium M.
    If s is a state of M, G[s] will provide a dictionary mapping
    the neighbors of s to the actions that produced those neighbors.
    """
    G = {S:{} for S in M}
    for S in M:
        for t in M.tokens():
            St = M(S,t)
            if St != S:
                if St in G[S]:
                    raise MediumError("multiple adjacency from %s to %s" % (S,St))
                G[S][St] = t
    return G


class LabeledGraphMedium(Medium):
    """
    A medium defined from an edge-labeled graph.
    The input graph G should have the property that G[v][w] is a token
    that transforms v to w. For instance,
    LabeledGraphMedium(StateTransitionGraph(M)) should result
    in a medium with the same behavior as M itself.
    """
    def __init__(self,G):
        if not isUndirected(G):
            raise MediumError("not an undirected graph")
        self._action = {v:{} for v in G}
        self._reverse = {}
        for v in G:
            for w in G[v]:
                t = G[v][w]
                if t in self._action[v]:
                    raise MediumError("multiple edges for state %s and token %s" % (v,t))
                self._action[v][t] = w
                if t not in self._reverse:
                    rt = G[w][v]
                    if rt in self._reverse:
                        raise MediumError("mismatched token reversals")
                    self._reverse[t] = rt
                    self._reverse[rt] = t
                elif G[w][v] != self._reverse[t]:
                    raise MediumError("mismatched token reversals")

    def states(self):
        return iter(self._action)

    def tokens(self):
        return iter(self._reverse)

    def reverse(self,t):
        return self._reverse[t]

    def action(self,S,t):
        return self._action[S].get(t,S)
    
    def __len__(self):
        return len(self._action)


def RoutingTable(M):
    """
    Return a dictionary mapping pairs (state1,state2) to tokens,
    such that the action of the token takes state1 closer to state2.
    By following successive tokens from this table, we can find
    a path in the medium that uses each token at most once
    and involves no token-reverse pairs.
    
    We use the O(n^2) time algorithm from arxiv:cs.DS/0206033.
    This is also a key step of the partial cube recognition algorithm
    from arxiv:0705.1025 -- as part of that algorithm, if we
    recognize that the input is not a medium, we raise MediumError.
    """
    G = StateTransitionGraph(M)
    current = initialState = iter(M).next()

    # find list of tokens that lead to the initial state
    activeTokens = set()
    for LG in BFS.BreadthFirstLevels(G,initialState):
        for v in LG:
            for w in LG[v]:
                activeTokens.add(G[w][v])
    for t in activeTokens:
        if M.reverse(t) in activeTokens:
            raise MediumError("shortest path to initial state is not concise")
    activeTokens = list(activeTokens)
    inactivated = object()  # flag object to mark inactive tokens

    # rest of data structure: point from states to list and list to states
    activeForState = {S:-1 for S in M}
    statesForPos = [[] for i in activeTokens]
    
    def scan(S):
        """Find the next token that is effective for s."""
        i = activeForState[S]
        while True:
            i += 1
            if i >= len(activeTokens):
                raise MediumError("no active token from %s to %s" %(S,current))
            if activeTokens[i] != inactivated and M(S,activeTokens[i]) != S:
                activeForState[S] = i
                statesForPos[i].append(S)
                return
    
    # set initial active states
    for S in M:
        if S != current:
            scan(S)

    # traverse the graph, maintaining active tokens
    visited = set()
    routes = {}
    for prev,current,edgetype in DFS.search(G,initialState):
        if prev != current and edgetype != DFS.nontree:
            if edgetype == DFS.reverse:
                prev,current = current,prev
            
            # add token to end of list, point to it from old state
            activeTokens.append(G[prev][current])
            activeForState[prev] = len(activeTokens) - 1
            statesForPos.append([prev])
            
            # inactivate reverse token, find new token for its states
            activeTokens[activeForState[current]] = inactivated
            for S in statesForPos[activeForState[current]]:
                if S != current:
                    scan(S)

            # remember routing table as part of returned results
            if current not in visited:
                for S in M:
                    if S != current:
                        routes[S,current] = activeTokens[activeForState[S]]

    return routes


def HypercubeEmbedding(M):
    """Map medium states isometrically onto a hypercube."""
    dim = 0
    tokmap = {}
    for t in M.tokens():
        if t not in tokmap:
            tokmap[t] = tokmap[M.reverse(t)] = 1<<dim
            dim += 1
    embed = {}
    G = StateTransitionGraph(M)
    for prev,current,edgetype in DFS.search(G):
        if edgetype == DFS.forward:
            if prev == current:
                embed[current] = 0
            else:
                embed[current] = embed[prev] ^ tokmap[G[prev][current]]
    return embed


# Perform some sanity checks if run standalone

class MediumTest(unittest.TestCase):

    # make medium from all five-bit numbers that have 2 or 3 bits lit
    twobits = [3,5,6,9,10,12,17,18,20,24]
    threebits = [31^x for x in twobits]
    M523 = BitvectorMedium(twobits+threebits,5)

    def testStates(self):
        """Check that iter(Medium) generates the correct set of states."""
        M = MediumTest.M523
        L1 = list(M)
        L1.sort()
        L2 = MediumTest.twobits + MediumTest.threebits
        L2.sort()
        self.assertEqual(L1,L2)
    
    def testTokens(self):
        """Check that Medium.tokens() generates the correct set of tokens."""
        M = MediumTest.M523
        toks = [(i,False) for i in range(5)] + [(i,True) for i in range(5)]
        self.assertEqual(set(toks),set(M.tokens()))
        for t in toks:
            i,b = t
            self.assertEqual(M.reverse(t),(i,not b))
    
    def testAction(self):
        """Check that the action of the tokens is what we expect."""
        M = MediumTest.M523
        for x in M:
            for i in range(5):
                b = (x>>i)&1
                self.assertEqual(x,M(x,(i,b)))
                y = M(x,(i,not b))
                if b == (x in MediumTest.twobits):
                    self.assertEqual(x,y)
                else:
                    self.assertEqual(y,x^(1<<i))
    
    def testRouting(self): 
        """Check that RoutingTable finds paths that decrease Hamming dist."""
        M = MediumTest.M523
        R = RoutingTable(M)
        for x in M:
            for y in M:
                if x != y:
                    i,b = R[x,y]
                    self.assertEqual((x^y)&(1<<i),1<<i)
                    self.assertEqual((x>>i)&1, not b)
    
    def testExplicit(self):            
        """Check that ExplicitMedium looks the same as its argument."""
        M = MediumTest.M523
        E = ExplicitMedium(M)
        self.assertEqual(set(M),set(E))
        self.assertEqual(set(M.tokens()),set(E.tokens()))
        for t in M.tokens():
            self.assertEqual(M.reverse(t),E.reverse(t))
        for s in M:
            for t in M.tokens():
                self.assertEqual(M(s,t),E(s,t))

    def testEmbed(self):
        """Check that HypercubeEmbedding finds appropriate coordinates."""
        M = MediumTest.M523
        E = HypercubeEmbedding(M)
        def ham(x,y):
            z = x^y
            d = 0
            while z:
                d += 1
                z &= z-1
            return d
        for x in M:
            for y in M:
                self.assertEqual(ham(x,y),ham(E[x],E[y]))     

    def testGraph(self):
        """Check that LabeledGraphMedium(StateTransitionGraph(M)) = M."""
        M = MediumTest.M523
        L = LabeledGraphMedium(StateTransitionGraph(M))
        self.assertEqual(set(M),set(L))
        self.assertEqual(set(M.tokens()),set(L.tokens()))
        for t in M.tokens():
            self.assertEqual(M.reverse(t),L.reverse(t))
        for s in M:
            for t in M.tokens():
                self.assertEqual(M(s,t),L(s,t))

if __name__ == "__main__":
    unittest.main()




"""Bipartite.py

Two-color graphs and find related structures.
D. Eppstein, May 2004.
"""

import unittest
from Biconnectivity import BiconnectedComponents
import Graphs
import DFS

class NonBipartite(Exception):
    pass

def TwoColor(G):
    """
    Find a bipartition of G, if one exists.
    Raises NonBipartite or returns dict mapping vertices
    to two colors (True and False).
    """
    color = {}
    for v,w,edgetype in DFS.search(G):
        if edgetype is DFS.forward:
            color[w] = not color.get(v,False)
        elif edgetype is DFS.nontree and color[v] == color[w]:
            raise NonBipartite
    return color

def Bipartition(G):
    """
    Find a bipartition of G, if one exists.
    Raises NonBipartite or returns sequence of vertices
    on one side of the bipartition.
    """
    color = TwoColor(G)
    for v in color:
        if color[v]:
            yield v

def isBipartite(G):
    """
    Return True if G is bipartite, False otherwise.
    """
    try:
        TwoColor(G)
        return True
    except NonBipartite:
        return False

def BipartiteOrientation(G,adjacency_list_type=set):
    """
    Given an undirected bipartite graph G, return a directed graph in which
    the edges are oriented from one side of the bipartition to the other.
    The second argument has the same meaning as in Graphs.copyGraph.
    """
    B = Bipartition(G)
    return {v:adjacency_list_type(iter(G[v])) for v in B}

def OddCore(G):
    """
    Subgraph of vertices and edges that participate in odd cycles.
    Aka, the union of nonbipartite biconnected components.
    """
    return Graphs.union(*[C for C in BiconnectedComponents(G)
                          if not isBipartite(C)])

# If run as "python Bipartite.py", run tests on various small graphs
# and check that the correct results are obtained.

class BipartitenessTest(unittest.TestCase):
    def cycle(self,n):
        return {i:[(i-1)%n,(i+1)%n] for i in range(n)}

    def testEvenCycles(self):
        for i in range(4,12,2):
            self.assertEqual(isBipartite(self.cycle(i)), True)

    def testOddCycles(self):
        for i in range(3,12,2):
            self.assertEqual(isBipartite(self.cycle(i)), False)

if __name__ == "__main__":
    unittest.main()   




"""PartialOrder.py

Various operations on partial orders and directed acyclic graphs.

D. Eppstein, July 2006.
"""

import unittest
from DFS import preorder,postorder
import BipartiteMatching

def isTopologicalOrder(G,L):
    """Check that L is a topological ordering of directed graph G."""
    vnum = {}
    for i in range(len(L)):
        if L[i] not in G:
            return False
        vnum[L[i]] = i
    for v in G:
        if v not in vnum:
            return False
        for w in G[v]:
            if w not in vnum or vnum[w] <= vnum[v]:
                return False
    return True

def TopologicalOrder(G):
    """Find a topological ordering of directed graph G."""
    L = list(postorder(G))
    L.reverse()
    if not isTopologicalOrder(G,L):
        raise ValueError("TopologicalOrder: graph is not acyclic.")
    return L

def isAcyclic(G):
    """Return True if G is a directed acyclic graph, False otherwise."""
    L = list(postorder(G))
    L.reverse()
    return isTopologicalOrder(G,L)

def TransitiveClosure(G):
    """
    The transitive closure of graph G.
    This is a graph on the same vertex set containing an edge (v,w)
    whenever v != w and there is a directed path from v to w in G.
    """
    TC = {v:set(preorder(G,v)) for v in G}
    for v in G:
        TC[v].remove(v)
    return TC

def TracePaths(G):
    """
    Turn a DAG with indegree and outdegree <= 1 into a sequence of lists.
    """
    L = []
    for v in TopologicalOrder(G):
        if L and v not in G[L[-1]]:
            yield L
            L = []
        L.append(v)
    if L:
        yield L

def MinimumPathDecomposition(G):
    """
    Cover a directed acyclic graph with a minimum number of paths.
    """
    M,A,B = BipartiteMatching.matching(G)
    H = {v:[] for v in G}
    for v in G:
        if v in M:
            H[M[v]] = (v,)
    return TracePaths(H)

def MinimumChainDecomposition(G):
    """
    Cover a partial order with a minimum number of chains.
    By Dilworth's theorem the number of chains equals the size
    of the largest antichain of the order. The input should be
    a directed acyclic graph, not necessarily transitively closed.
    """
    return MinimumPathDecomposition(TransitiveClosure(G))
    
def MaximumAntichain(G):
    """
    Find a maximum antichain in the given directed acyclic graph.
    """
    if not isAcyclic(G):
        raise ValueError("MaximumAntichain: input is not acyclic.")
    TC = TransitiveClosure(G)
    M,A,B = BipartiteMatching.matching(TransitiveClosure(G))
    return set(A).intersection(B)

class PartialOrderTest(unittest.TestCase):
    cube = {i:[i^b for b in (1,2,4,8) if i^b > i] for i in range(16)}
            
    def testHypercubeAcyclic(self):
        self.assert_(isAcyclic(self.cube))
        
    def testHypercubeClosure(self):
        TC = TransitiveClosure(self.cube)
        for i in range(16):
            self.assertEqual(TC[i],
                {j for j in range(16) if i & j == i and i != j})

    def testHypercubeAntichain(self):        
        A = MaximumAntichain(self.cube)
        self.assertEqual(A,{3,5,6,9,10,12})
        
    def testHypercubeDilworth(self):
        CD = list(MinimumChainDecomposition(self.cube))
        self.assertEqual(len(CD),6)

if __name__ == "__main__":
    unittest.main()   



"""readUndirectedGraph.py

Parse various undirected graph formats

readUndirectedGraph(arg) takes an argument which may be a file object,
string, or list of strings, and returns a parsed undirected graph.

Various input formats are supported and automatically detected:

 - MALF, edge list, and node edge list format documentation is at
   http://loki.cs.brown.edu:8081/graphserver/gds/formats.shtml

 - GraphML format doc is at http://graphml.graphdrawing.org/

 - graph6 and sparse6 doc is at http://cs.anu.edu.au/~bdm/data/formats.txt

 - LEDA.GRAPH documentation is at
   http://www.algorithmic-solutions.info/leda_guide/graphs/leda_native_graph_fileformat.html

The output is a dict mapping vertices to adjacency lists (also dicts).
Each vertex is either a unique integer or a string.
Each adjacency list maps neighbors to edge ids, either unique integers or strings.
Each edge is represented twice, once for each endpoint.

Only the graph structure itself is returned; any additional information may be lost.

D. Eppstein, UC Irvine, August 12, 2003.
"""

class GraphFormatError(Exception):
	pass

def graphNum(s):
	"""Parse s as an integer, complain appropriately if it fails."""
	try:
		return int(s)
	except:
		raise GraphFormatError, 'Number expected: "%s"' % s

def graph():
	"""Create a new empty graph."""
	return {}
	
def vertex(G, v):
	"""Add new vertex v to graph G."""
	if v in G:
		raise GraphFormatError, 'Duplicate vertex %s', str(v)
	G[v] = {}

def edge(G,u,v,e):
	"""Add edge e connecting vertices u and v in graph G.  Vertices must already be in G."""
	if u == v:
		raise GraphFormatError, 'Self-loop at %s' % str(u)
	if u not in G:
		raise GraphFormatError, 'Unexpected vertex %s in edge to %s' % (str(u),str(v))
	if v not in G:
		raise GraphFormatError, 'Unexpected vertex %s in edge from %s' % (str(v),str(u))
	G[u][v] = G[v][u] = e


# ==========================================================================
#		MALF format
# ==========================================================================

def readMALF(lines):
	"""Read undirected graph in MALF format."""
	G = graph()
	lines = iter(filter(None,lines))
	for line in lines:
		if line == '#':
			break
		n = graphNum(line.split()[0])
		if n != len(G)+1:
			raise GraphFormatError, 'Nonconsecutive vertices in MALF'
		vertex(G,n)

	m = 0
	for line in lines:
		nums = [graphNum(x) for x in line.split()]
		m += 1
		if len(nums) != 4:
			raise GraphFormatError, "Other than four numbers per line in MALF edge list"
		elif nums[0] != m:
			raise GraphFormatError, 'Nonconsecutive edges in MALF'
		elif nums[1] != 0:
			raise GraphFormatError, "Unrecognized edge type in MALF edge list"
		edge(G,nums[2],nums[3],m)

	return G


# ==========================================================================
#		Edge list format
# ==========================================================================
	
def readEdgeList(lines):
	"""Read undirected graph in edge list format."""
	G = graph()
	m = 0
	for line in filter(None,lines):
		words = line.split()
		if len(words) < 2 or len(words) > 3:
			raise GraphFormatError, 'Wrong number of words in edge list: "%s"' % line
		if len(words) == 3 and words[1] != '-':
			raise GraphFormatError, 'Unrecognized edge type "%s" in edge list' % words[1]
		u, v = words[0], words[-1]
		if u not in G:
			vertex(G, u)
		if v not in G:
			vertex(G, v)
		m = m + 1
		edge(G, u, v, m)

	return G


# ==========================================================================
#		Node edge list format
# ==========================================================================
	
def readNodeEdgeList(lines):
	"""Read undirected graph in node edge list format."""
	G = graph()
	EdgeNames = {}
	lines = iter(filter(None,lines))
	numEdges = [0]

	def addVertex(line):
		vertex(G, line)

	def addEdge(line, id):
		u = line
		v = lines.next()
		if v.startswith('//'):
			raise GraphFormatError, 'Missing edge endpoint in node edge list'
		edge(G, u, v, id)
		numEdges[0] += 1

	def anonEdge(line):
		return addEdge(line, numEdges[0]+1)
		
	def namedEdge(line):
		id = line
		if id in EdgeNames:
			raise GraphFormatError, 'Edge name "%s" used twice in node edge list' % id
		addEdge(lines.next(), id)
		EdgeNames[id] = id
		
	def noActionYet(line):
		raise GraphFormatError, 'No section yet in node edge list'
		
	actions = {
		'nodes': addVertex,
		'edges': anonEdge,
		'named edges': namedEdge,
	}
	action = noActionYet
	
	for line in lines:
		if line.startswith('// '):
			try:
				action = actions[line[3:]]
			except KeyError:
				raise GraphFormatError, 'Unrecognized section "%s" in node edge list' % line[3:]
		else:
			action(line)

	return G


# ==========================================================================
#		GraphML format
# ==========================================================================
	
def readGraphML(lines):
	"""Read undirected graph in GraphML format."""
	context = []
	G = graph()
	edgecounter = [0]
	defaultDirectedness = ['true']
	
	def start_element(name,attrs):
		context.append(name)
		if len(context) == 1:
			if name != 'graphml':
				raise GraphFormatError, 'Unrecognized outer tag "%s" in GraphML' % name
		elif len(context) == 2 and name == 'graph':
			if 'edgedefault' not in attrs:
				raise GraphFormatError, 'Required attribute edgedefault missing in GraphML'
			if attrs['edgedefault'] == 'undirected':
				defaultDirectedness[0] = 'false'
		elif len(context) == 3 and context[1] == 'graph' and name == 'node':
			if 'id' not in attrs:
				raise GraphFormatError, 'Anonymous node in GraphML'
			vertex(G, attrs['id'])
		elif len(context) == 3 and context[1] == 'graph' and name == 'edge':
			if 'source' not in attrs:
				raise GraphFormatError, 'Edge without source in GraphML'
			if 'target' not in attrs:
				raise GraphFormatError, 'Edge without target in GraphML'
			if attrs.get('directed', defaultDirectedness[0]) != 'false':
				raise GraphFormatError, 'Directed edge in GraphML'
			edge(G, attrs['source'], attrs['target'], edgecounter[0])
			edgecounter[0] += 1			
		
	def end_element(name):
		context.pop()
	
	import xml.parsers.expat
	p = xml.parsers.expat.ParserCreate()
	p.StartElementHandler = start_element
	p.EndElementHandler = end_element
	for line in lines:
		p.Parse(line)
	p.Parse("", 1)
	return G


# ==========================================================================
#		Graph6 and Sparse6 format
# ==========================================================================
	
def graph6data(str):
	"""Convert graph6 character sequence to 6-bit integers."""
	v = [ord(c)-63 for c in str]
	if min(v) < 0 or max(v) > 63:
		return None
	return v
	
def graph6n(data):
	"""Read initial one or four-unit value from graph6 sequence.  Return value, rest of seq."""
	if data[0] <= 62:
		return data[0], data[1:]
	return (data[1]<<12) + (data[2]<<6) + data[3], data[4:]

def readGraph6(str):
	"""Read undirected graph in graph6 format."""
	if str.startswith('>>graph6<<'):
		str = str[10:]
	data = graph6data(str)
	n, data = graph6n(data)
	nd = (n*(n-1)//2 + 5) // 6
	if len(data) != nd:
		raise GraphFormatError, 'Expected %d bits but got %d in graph6' % (n*(n-1)//2, len(data)*6)

	def bits():
		"""Return sequence of individual bits from 6-bit-per-value list of data values."""
		for d in data:
			for i in [5,4,3,2,1,0]:
				yield (d>>i)&1
				
	nEdges = 0
	G = graph()
	for i in range(n):
		vertex(G, i)

	for (i,j),b in zip([(i,j) for j in range(1,n) for i in range(j)], bits()):
		if b:
			edge(G, i, j, nEdges)
			nEdges += 1

	return G
					
def readSparse6(str):
	"""Read undirected graph in sparse6 format."""
	if str.startswith('>>sparse6<<'):
		str = str[10:]
	if not str.startswith(':'):
		raise GraphFormatError, 'Expected colon in sparse6'
	n, data = graph6n(graph6data(str[1:]))
	k = 1
	while 1<<k < n:
		k += 1
	
	def parseData():
		"""Return stream of pairs b[i], x[i] for sparse6 format."""
		chunks = iter(data)
		d = None	# partial data word
		dLen = 0	# how many unparsed bits are left in d

		while 1:
			if dLen < 1:
				d = chunks.next()
				dLen = 6
			dLen -= 1
			b = (d>>dLen) & 1	# grab top remaining bit
			
			x = d & ((1<<dLen)-1) 	# partially built up value of x
			xLen = dLen				# how many bits included so far in x
			while xLen < k:			# now grab full chunks until we have enough
				d = chunks.next()
				dLen = 6
				x = (x<<6) + d
				xLen += 6
			x = (x >> (xLen - k))	# shift back the extra bits
			dLen = xLen - k
			yield b,x
	
	v = 0
	G = graph()
	nEdges = 0
	for i in range(n):
		vertex(G, i)

	for b,x in parseData():
		if b: v += 1
		if x >= n: break			# padding with ones can cause overlarge number here
		elif x > v: v = x
		else:
			edge(G, x, v, nEdges)
			nEdges += 1

	return G


# ==========================================================================
#		LEDA.GRAPH format
# ==========================================================================
	
def ledaLines(lines):
	"""Filter sequence of lines to keep only the relevant ones for LEDA.GRAPH"""
	def relevant(line):
		return line and not line.startswith('#')
	return filter(relevant, lines)
	
def readLeda(lines):
	"""Parse filtered LEDA.GRAPH format file."""
	lines = iter(lines)
	for i in range(3):
		lines.next()			# skip header lines
	G = graph()

	n = graphNum(lines.next())	# number of vertices
	for i in range(n):
		vertex(G, i+1)
		lines.next()			# skip LEDA data
	
	m = graphNum(lines.next())	# number of edges
	for i in range(m):
		words = lines.next().split()
		if len(words) < 4:
			raise GraphFormatError, 'Too few fields in LEDA.GRAPH edge %d' % (i+1)
		source = graphNum(words[0])
		target = graphNum(words[1])
		reversal = graphNum(words[2])
		if not reversal:
			raise GraphFormatError, 'Edge %d is directed in LEDA.GRAPH' % (i+1)
		if source < target:
			edge(G, source, target, i+1)
	
	return G


# ==========================================================================
#		Main entry
# ==========================================================================

def readUndirectedGraph(arg):
	"""Parse graph and return in modified GvR format.
	Argument may be a file object, a single string, or a sequence of lines.
	"""
	if isinstance(arg,file):
		lines = [L.strip() for L in arg]
	elif isinstance(arg,str) or isinstance(arg,unicode):
		lines = [arg]
	else:
		lines = list(arg)

	# Test out different possible formats,
	# ordered from more distinctive to more ambiguous.
	
	# Graph6 and Sparse6
	if len(lines) == 1:
		line = lines[0]
		if line.startswith('>>graph6<<') or graph6data(line):
			return readGraph6(line)
		elif line.startswith('>>sparse6<<') or \
				(line.startswith(':') and graph6data(line[1:])):
			return readSparse6(line)
			
	# LEDA.GRAPH
	leda = ledaLines(lines)
	if leda and leda[0] == 'LEDA.GRAPH':
		return readLeda(leda)

	# GraphML
	if lines[0].startswith("<"):
		return readGraphML(lines)
		
	# Node edge list
	if lines[0].startswith("//"):
		return readNodeEdgeList(lines)
		
	# MALF
	if '#' in lines:
		return readMALF(lines)

	# Edge list
	return readEdgeList(lines)



"""Biconnectivity.py

DFS-based algorithm for computing biconnected components.

D. Eppstein, April 2004.
"""

import unittest
from Graphs import isUndirected
from Util import arbitrary_item
from PartialOrder import TopologicalOrder

import DFS

disconnected = object() # flag for BiconnectedComponents

class BiconnectedComponents(DFS.Searcher):
    """
    Generate the biconnected components of G.  G should be represented in
    such a way that "for v in G" loops through the vertices, and "G[v]"
    produces a list of the neighbors of v; for instance, G may be a
    dictionary mapping each vertex to its neighbor set.
    The result of BiconnectedComponents(G) is a sequence of subgraphs of G.
    """

    def __init__(self,G):
        """Search for biconnected components of graph G."""
        if not isUndirected(G):
            raise ValueError("BiconnectedComponents: input not undirected graph")

        # set up data structures for DFS
        self._components = []
        self._dfsnumber = {}
        self._activelen = {}
        self._active = []
        self._low = {}
        self._ancestors = {} # directed subgraph from nodes to DFS ancestors

        # perform the Depth First Search
        DFS.Searcher.__init__(self,G)

        # clean up now-useless data structures
        del self._dfsnumber, self._activelen, self._active
        del self._low, self._ancestors

    def __iter__(self):
        """Return iterator for sequence of biconnected components."""
        return iter(self._components)

    def preorder(self,parent,child):
        if parent == child:
            self._active = [child]
        else:
            self._active.append(child)
        self._low[child] = self._dfsnumber[child] = len(self._dfsnumber)
        self._ancestors[child] = set()
        self._activelen[child] = len(self._active)

    def backedge(self,source,destination):
        if self._dfsnumber[destination] < self._dfsnumber[source]:
            self._low[source] = min(self._low[source],
                                    self._dfsnumber[destination])
            self._ancestors[source].add(destination)

    def postorder(self,parent,child):
        if self._low[child] != self._dfsnumber[parent]:
            self._low[parent] = min(self._low[parent],self._low[child])
            self._activelen[parent] = len(self._active)
        elif parent != child:
            self._component(self._activelen[parent],parent)
        elif not self._components or child not in self._components[-1]:
            self._component()

    def _component(self,start=0, articulation_point=disconnected):
        """Make new component, removing active vertices from start onward."""
        component = {}
        if articulation_point is not disconnected:
            component[articulation_point] = set()
        for v in self._active[start:]:
            component[v] = set()
            for w in self._ancestors[v]:
                component[v].add(w)
                component[w].add(v)
        del self._active[start:]
        self._components.append(component)


class NotBiconnected(Exception): pass

class BiconnectivityTester(DFS.Searcher):
    """
    Stripped down version of BiconnectedComponents.
    Either successfully inits or raises NotBiconnected.
    Otherwise does nothing.
    """

    def __init__(self,G):
        """Search for biconnected components of graph G."""
        if not isUndirected(G):
            raise NotBiconnected
        self._dfsnumber = {}
        self._low = {}
        self._rootedge = None
        DFS.Searcher.__init__(self,G)

    def preorder(self,parent,child):
        if parent == child and self._rootedge:
            raise NotBiconnected    # two roots, not even connected
        elif not self._rootedge and parent != child:
            self._rootedge = (parent,child)
        self._low[child] = self._dfsnumber[child] = len(self._dfsnumber)

    def backedge(self,source,destination):
        self._low[source] = min(self._low[source],self._dfsnumber[destination])

    def postorder(self,parent,child):
        if self._low[child] != self._dfsnumber[parent]:
            self._low[parent] = min(self._low[parent],self._low[child])
        elif (parent,child) == self._rootedge:
            pass                    # end of first component, >1 vertices
        elif parent != child:
            raise NotBiconnected    # articulation point
        elif not self._rootedge:
            self._rootedge = parent,child   # end of first component, isolani


def isBiconnected(G):
    """Return True if graph G is biconnected, False otherwise."""
    try:
        BiconnectivityTester(G)
        return True
    except NotBiconnected:
        return False


class stOrienter(DFS.Searcher):
    """
    Subclass for st-orienting a biconnected graph.
    """

    def __init__(self,G):
        """Relate edges for st-orientation."""
        if not isUndirected(G):
            raise ValueError("stOrienter: input not undirected graph")

        # set up data structures for DFS
        self._dfsnumber = {}
        self._low = {}
        self._down = {} # down[v] = child we're currently exploring from v
        self._lowv = {} # lowv[n] = vertex with low number n
        
        # The main data structure!
        # a dictionary mapping edges to lists of edges
        # each of which should be oriented the same as the key.
        self.orient = {}
        self.roots = [] # edges with no predecessor

        # perform the Depth First Search
        DFS.Searcher.__init__(self,G)

        # clean up now-useless data structures
        del self._dfsnumber, self._low, self._down, self._lowv

    def __iter__(self):
        """Return iterator for sequence of biconnected components."""
        return iter(self._components)

    def preorder(self,parent,child):
        self._low[child] = self._dfsnumber[child] = len(self._dfsnumber)
        self._lowv[self._low[child]] = self._down[parent] = child

    def backedge(self,source,destination):
        if self._dfsnumber[destination] < self._dfsnumber[source]:
            self._low[source] = min(self._low[source],
                                    self._dfsnumber[destination])
            if source != self._down[destination]:
                self.addOrientation(destination,source,destination)

    def postorder(self,parent,child):
        if self._low[child] != self._dfsnumber[parent]:
            self._low[parent] = min(self._low[parent],self._low[child])
            self.addOrientation(child,parent,self._lowv[self._low[child]])
        elif parent != child:
            self.roots.append((parent,child))

    def addOrientation(self,source,dest,anchor):
        """Store orientation info for source->dest edge.
        It should be oriented the same as the edge from the anchor
        to the current child of the anchor."""
        child = self._down[anchor]
        L = self.orient.setdefault((anchor,child),[])
        L.append((source,dest))

def stOrientation(G):
    """Find an acyclic orientation of G, with one source and one sink."""
    stO = stOrienter(G)
    if len(stO.roots) != 1:
        raise NotBiconnected

    source,dest = stO.roots[0]
    G = {v:set() for v in G}
    orientable = []

    while True:
        G[source].add(dest)
        for u,v in stO.orient.get((source,dest),[]):
            orientable.append((u,v))
        for v,u in stO.orient.get((dest,source),[]):
            orientable.append((u,v))
        if not orientable:
            break
        source,dest = orientable.pop()
    
    return G

# If run as "python Biconnectivity.py", run tests on various small graphs
# and check that the correct results are obtained.

class BiconnectivityTest(unittest.TestCase):
    G1 = {
        0: [1,2,5],
        1: [0,5],
        2: [0,3,4],
        3: [2,4,5,6],
        4: [2,3,5,6],
        5: [0,1,3,4],
        6: [3,4],
    }
    G2 = {
        0: [2,5],
        1: [3,8],
        2: [0,3,5],
        3: [1,2,6,8],
        4: [7],
        5: [0,2],
        6: [3,8],
        7: [4],
        8: [1,3,6],
    }

    def testIsBiconnected(self):
        """G1 is biconnected but G2 is not."""
        self.assertEqual(isBiconnected(self.G1), True)
        self.assertEqual(isBiconnected(self.G2), False)

    def testBiconnectedComponents(self):
        """G2 has four biconnected components."""
        C = BiconnectedComponents(self.G2)
        CV = [component.keys() for component in C]
        for comp in CV:
            comp.sort()
        CV.sort()
        self.assertEqual(CV,[[0,2,5],[1,3,6,8],[2,3],[4,7]])
    
    def testSTOrientation(self):
        STO = stOrientation(self.G1)
        L = list(TopologicalOrder(STO))
        indegree = dict([(v,0) for v in self.G1])
        for v in L:
            for w in STO[v]:
                indegree[w] += 1
        outdegree = dict([(v,len(STO[v])) for v in self.G1])
        self.assertEqual(len([v for v in self.G1 if indegree[v] == 0]), 1)
        self.assertEqual(len([v for v in self.G1 if outdegree[v] == 0]), 1)

if __name__ == "__main__":
    unittest.main()   




"""MinimumSpanningTree.py

Kruskal's algorithm for minimum spanning trees. D. Eppstein, April 2006.
"""

from UnionFind import UnionFind

def MinimumSpanningTree(G):
    """
    Return the minimum spanning tree of an undirected graph G.
    G should be represented in such a way that G[u][v] gives the
    length of edge u,v, and G[u][v] should always equal G[v][u].
    The tree is returned as a list of edges.
    """
    
    # Kruskal's algorithm: sort edges by weight, and add them one at a time.
    # We use Kruskal's algorithm, first because it is very simple to
    # implement once UnionFind exists, and second, because the only slow
    # part (the sort) is sped up by being built in to Python.
    subtrees = UnionFind()
    tree = []
    edges = [(G[u][v],u,v) for u in G for v in G[u]]
    edges.sort()
    for W,u,v in edges:
        if subtrees[u] != subtrees[v]:
            tree.append((u,v))
            subtrees.union(u,v)
    return tree        



"""LongestIncreasingSubsequence.py

Find longest increasing subsequence of an input sequence.
D. Eppstein, April 2004
"""

import unittest
from bisect import bisect_left

def LongestIncreasingSubsequence(S):
    """
    Find and return longest increasing subsequence of S.
    If multiple increasing subsequences exist, the one that ends
    with the smallest value is preferred, and if multiple
    occurrences of that value can end the sequence, then the
    earliest occurrence is preferred.
    """

    # The main data structures: head[i] is value x from S that
    # terminates a length-i subsequence, and tail[i] is either
    # None (if i=0) or the pair (head[i-1],tail[i-1]) as they
    # existed when x was processed.
    head = []
    tail = [None]

    for x in S:
        i = bisect_left(head,x)
        if i >= len(head):
            head.append(x)
            if i > 0:
                tail.append((head[i-1],tail[i-1]))
        elif head[i] > x:
            head[i] = x
            if i > 0:
                tail[i] = head[i-1],tail[i-1]

    if not head:
        return []

    output = [head[-1]]
    pair = tail[-1]
    while pair:
        x,pair = pair
        output.append(x)

    output.reverse()
    return output

# If run as "python LongestIncreasingSubsequence.py", run tests on various
# small lists and check that the correct subsequences are generated.

class LISTest(unittest.TestCase):
    def testLIS(self):
        self.assertEqual(LongestIncreasingSubsequence([]),[])
        self.assertEqual(LongestIncreasingSubsequence(range(10,0,-1)),[1])
        self.assertEqual(LongestIncreasingSubsequence(range(10)),range(10))
        self.assertEqual(LongestIncreasingSubsequence([3,1,4,1,5,9,2,6,5,3,5,8,9,7,9]),
                                                      [1,2,3,5,8,9])

if __name__ == "__main__":
    unittest.main()   



"""Chordal.py

Recognize and compute elimination ordering of chordal graphs, using
an algorithm from Habib, McConnell, Paul, and Viennot, "Lex-BFS and
Partition Refinement, with Applications to Transitive Orientation,
Interval Graph Recognition, and Consecutive Ones Testing", Theor.
Comput. Sci. 234:59-84 (2000), http://www.cs.colostate.edu/~rmm/lexbfs.ps

D. Eppstein, November 2003.
"""

import unittest
from LexBFS import LexBFS
from Graphs import isUndirected

def PerfectEliminationOrdering(G):
    """Return a perfect elimination ordering, or raise an exception if not chordal.
    G should be represented in such a way that "for v in G" loops through
    the vertices, and "G[v]" produces a list of the neighbors of v; for
    instance, G may be a dictionary mapping each vertex to its neighbor set.
    Running time is O(n+m) and additional space usage over G is O(n+m).
    """
    alreadyProcessed = set()
    B = list(LexBFS(G))
    position = {B[i]:i for i in range(len(B))}
    leftNeighbors = {}
    parent = {}
    for v in B:
        leftNeighbors[v] = set(G[v]) & alreadyProcessed
        alreadyProcessed.add(v)
        if leftNeighbors[v]:
            parent[v] = B[max([position[w] for w in leftNeighbors[v]])]
            if not leftNeighbors[v] - {parent[v]} <= leftNeighbors[parent[v]]:
                raise ValueError("Input to PerfectEliminationOrdering is not chordal")
    B.reverse()
    return B

def Chordal(G):
    """Test if a given graph is chordal."""
    if not isUndirected(G):
        raise ValueError("Input to Chordal is not an undirected graph")
    try:
        PerfectEliminationOrdering(G)
    except:
        return False
    return True

class ChordalTest(unittest.TestCase):
    claw = {0:[1,2,3],1:[0],2:[0],3:[0]}
    butterfly = {0:[1,2,3,4],1:[0,2],2:[0,1],3:[0,4],4:[0,3]}
    diamond = {0:[1,2],1:[0,2,3],2:[0,1,3],3:[1,2]}
    quad = {0:[1,3],1:[0,2],2:[1,3],3:[0,2]}
    graphs = [(claw,True), (butterfly,True), (diamond,True), (quad,False)]
    
    def testChordal(self):
        """Check that Chordal() returns the correct answer on each test graph."""
        for G,isChordal in ChordalTest.graphs:
            self.assertEqual(Chordal(G), isChordal)

    def testElimination(self):
        """Check that PerfectEliminationOrdering generates an elimination ordering."""
        for G,isChordal in ChordalTest.graphs:
            if isChordal:
                eliminated = set()
                for v in PerfectEliminationOrdering(G):
                    eliminated.add(v)
                    for w in G[v]:
                        for x in G[v]:
                            if w != x and w not in eliminated and x not in eliminated:
                                self.assert_(w in G[x] and x in G[w]) 

if __name__ == "__main__":
    unittest.main()   



"""DFS.py

Algorithms for depth first search in Python.
We need to search iteratively rather than recursively in
order to avoid Python's low recursion limit.

D. Eppstein, July 2004.
"""

# Types of edges in DFS traversal.
# The numerical values are used in DepthFirstSearcher, change with care.
forward = 1     # traversing edge (v,w) from v to w
reverse = -1    # returning backwards on (v,w) from w to v
nontree = 0     # edge (v,w) is not part of the DFS tree

whole_graph = object()  # special flag object, do not use as a graph vertex

def search(G,initial_vertex = whole_graph):
    """
    Generate sequence of triples (v,w,edgetype) for DFS of graph G.
    The subsequence for each root of each tree in the DFS forest starts
    with (root,root,forward) and ends with (root,root,reverse).
    If the initial vertex is given, it is used as the root and vertices
    not reachable from it are not searched.
    """
    visited = set()
    if initial_vertex == whole_graph:
        initials = G
    else:
        initials = [initial_vertex]
    for v in initials:
        if v not in visited:
            yield v,v,forward
            visited.add(v)
            stack = [(v,iter(G[v]))]
            while stack:
                parent,children = stack[-1]
                try:
                    child = children.next()
                    if child in visited:
                        yield parent,child,nontree
                    else:
                        yield parent,child,forward
                        visited.add(child)
                        stack.append((child,iter(G[child])))
                except StopIteration:
                    stack.pop()
                    if stack:
                        yield stack[-1][0],parent,reverse
            yield v,v,reverse

def preorder(G,initial_vertex = whole_graph):
    """Generate all vertices of graph G in depth-first preorder."""
    for v,w,edgetype in search(G,initial_vertex):
        if edgetype is forward:
            yield w

def postorder(G,initial_vertex = whole_graph):
    """Generate all vertices of graph G in depth-first postorder."""
    for v,w,edgetype in search(G,initial_vertex):
        if edgetype is reverse:
            yield w

def reachable(G,v,w):
    """Can we get from v to w in graph G?"""
    return w in preorder(G,v)

class Searcher:
    """
    Handler for performing general depth first searches of graphs.
    Some or all of the routines preorder, postorder, and backedge
    should be shadowed in order to make the search do something useful.
    """

    def preorder(self,parent,child):
        """
        Called when DFS visits child, before visiting all grandchildren.
        Parent==child when child is the root of each DFS tree.
        """
        pass

    def postorder(self,parent,child):
        """
        Called when DFS visits child, after visiting all grandchildren.
        Parent==child when child is the root of each DFS tree.
        """
        pass

    def backedge(self,source,destination):
        """Called when DFS discovers an edge to a non-child."""
        pass

    def __init__(self,G):
        """Perform a depth first search of graph G."""
        dispatch = [self.backedge,self.preorder,self.postorder]
        for v,w,edgetype in search(G):
            dispatch[edgetype](v,w)



from distutils.core import setup

setup(
    name='PADS',
    version='0.0.20131119',
    author='David Eppstein',
    author_email='eppstein@ics.uci.edu',
    maintainer='Tim Pederick',
    maintainer_email='pederick@gmail.com',
    packages=['pads'],
    url='http://pypi.python.org/pypi/PADS/',
    description='Python Algorithms and Data Structures',
    classifiers=['Intended Audience :: Developers',
                 'License :: Public Domain',
                 'Programming Language :: Python',
                 'Topic :: Software Development'],
    long_description=open('README.txt').read(),
)




"""xyzGraph.py

An xyz graph is the graph formed by a set of points in R^3 such that any
axis-parallel line intersects the set in zero or two points; we add an edge
for the two points on any nonempty line. These graphs include all bipartite
simple polyhedra, as well as some other nonplanar graphs; see
http://11011110.livejournal.com/tag/xyz+graphs for more.

We implement here an exponential-time algorithm for finding such
representations of an arbitrary graph, when they exist.

D. Eppstein, June 2006.
"""

from Graphs import isUndirected
from PartialOrder import TopologicalOrder
from StrongConnectivity import StronglyConnectedComponents
from Biconnectivity import stOrientation
import unittest
from collections import defaultdict

def CubicMatchPartitions(G):
    """Partition a biconnected cubic graph G into three matchings.
    Each matching is represented as a graph, in which G[v] is a list
    of the three edges of G in the order of the three matchings.
    This function generates a sequence of such representations.
    """
    
    if not isUndirected(G):
        raise ValueError("CubicMatchPartitions: graph is not undirected")
    for v in G:
        if len(G[v]) != 3:
            raise ValueError("CubicMatchPartitions: graph is not cubic")
    ST = stOrientation(G)
    L = TopologicalOrder(ST)
    for B in xrange(1L<<(len(L)//2 - 1)):
        # Here with a bitstring representing the sequence of choices
        out = {}
        pos = 0
        for v in L:
            source = [w for w in G[v] if w in out]
            sourcepos = {}
            adjlist = [None,None,None]
            for w in source:
                sourcepos[w] = [i for i in (0,1,2) if out[w][i]==v][0]
                adjlist[sourcepos[w]] = w
            usedpos = [sourcepos[w] for w in source]
            if len(set(usedpos)) != len(usedpos):
                # two edges in with same index, doesn't form matching
                break 
            elif len(source) == 0:
                # start vertex, choose one orientation
                adjlist = list(ST[v])
            elif len(source) == 1:
                # two outgoing vertices, one incoming
                avail = [i for i in (0,1,2) if i != usedpos[0]]
                if B & (1L<<pos):
                    avail.reverse()
                pos += 1
                for i,w in zip(avail,list(ST[v])):
                    adjlist[i] = w
            elif len(source) == 2:
                avail = 3 - sum(usedpos)
                adjlist[avail] = list(ST[v])[0]
            out[v] = adjlist
            if len(source) == 3:
                # final vertex of topological ordering, still all consistent
                yield out
                
def groupByCycles(G,i,j):
    """
    Collect cycles of G[v][i] and G[v][j] edges,
    return a dictionary mapping v to the number of its cycle.
    """
    G = {v:(G[v][i],G[v][j]) for v in G}
    index = 0
    D = {}
    for C in StronglyConnectedComponents(G):
        for v in C:
            D[v] = index
        index += 1
    return D

def isxyz(points):
    """
    True if there are two points per axis-parallel line, False otherwise.
    """
    for i,j in [(0,1),(0,2),(1,2)]:
        projections = defaultdict(list)
        for p in points:
            projections[p[i],p[j]].append(p)
        for L in projections.itervalues():
            if len(L) != 2:
                return False
    return True
        

def xyzEmbeddings(G):
    """
    List all xyz graph embeddings of G.
    Each is returned as a dictionary mapping vertices to triples of points.
    To get just the points, use D.itervalues().
    """
    for G in CubicMatchPartitions(G):
        xyz = [groupByCycles(G,i,j) for i,j in [(0,1),(0,2),(1,2)]]
        xyz = {v:[xyz[i][v] for i in (0,1,2)] for v in G}
        if isxyz(list(xyz.itervalues())):
            yield xyz

class xyzGraphTest(unittest.TestCase):
    cube = {v:[v^i for i in (1,2,4)] for v in range(8)}

    def testCubicMatchPartitions(self):
        """Check that a cube has the right number of matching partitions."""
        self.assertEqual(len(list(CubicMatchPartitions(self.cube))),4)

    def testCubeIsXYZ(self):
        """Check that a cube is correctly identified as an xyz graph."""
        self.assertEqual(len(list(xyzEmbeddings(self.cube))),1)

if __name__ == "__main__":
    unittest.main()   

